#!/bin/bash

# Filename: smtt-write-application-key.sh
# Synopsis: Script that stores the application INIT_KEY into
#           the squashfs filesystems of the USB.
# Author: Luis Dalo
# Version: 0.0.2
# Last Edited: 9 Nov 2017

set -e

###############
#  VARIABLES  #
###############

HELP="Usage: $0  [PATH_TO_USB]  [PATH_TO_PREP_FILE]\n
where:\n
- PATH_TO_USB: Absolute path of the USB where to install the OS.\n
- PATH_TO_PREP_FILE: Absolute path where the swap.prep file is located.\n"

##########################
#  VALIDATING ARGUMENTS  #
##########################

if [ $# -ne 2 ]
then
    echo "E: Wrong usage."
    echo -e ${HELP}
    exit 1
else
    USB_PATH=$1
    PATH_TO_PREP=$2
fi

if [ ! -d ${USB_PATH} ]
then
    echo "E: Not a valid USB Path (${USB_PATH})."
    echo -e ${HELP}
    exit 1
fi

if [ ! -d ${PATH_TO_PREP} ]
then
    echo "E: Not a valid PATH_TO_PREP_FILE (${PATH_TO_PREP})."
    echo -e ${HELP}
    exit 1
fi

if [ "${USB_PATH}" == "/" ]
then
    echo "E: PATH_TO_USB cannot be the root path."
    echo -e ${HELP}
    exit 1
fi

if [ ! -f ${PATH_TO_PREP}/swap.prep ]
then
    echo "E: Path ${PATH_TO_PREP} does not contain a swap.prep file"
    echo -e ${HELP}
    exit 1
fi

if [ $(find ${USB_PATH}/live/ -type f -name "filesystem*" | wc -l) -eq 0 ]
then
    echo "E: Path ${USB_PATH}/live/ does not contain a squashfs filesystem."
    echo -e ${HELP}
    exit 1
fi

#######################
#  DYNAMIC VARIABLES  #
#######################

SWAP_FILE=${PATH_TO_PREP}/swap.prep
SQUASHFS_ROOT_DIR=${PATH_TO_PREP}/squashfs-root

###########
#  BEGIN  #
###########

for filesystem in ${USB_PATH}/live/*.squashfs
do
    echo "I: Extracting the $(basename ${filesystem})"
    rm -rf ${SQUASHFS_ROOT_DIR}
    unsquashfs -d ${SQUASHFS_ROOT_DIR} ${filesystem}
    echo "I: Storing the INIT_KEY into the $(basename ${filesystem})"
    cp ${SWAP_FILE} ${SQUASHFS_ROOT_DIR}/etc/election.d/config/
    echo "I: Repacking the $(basename ${filesystem})"
    mksquashfs ${SQUASHFS_ROOT_DIR} ${PATH_TO_PREP}/filesystem.squashfs -noappend
    cp ${PATH_TO_PREP}/filesystem.squashfs ${filesystem}
    sync
    echo "I: Done for the $(basename ${filesystem})"
done

echo "I: Done updating swap.prep file on all filesystems."

##############
#  CLEAN UP  #
##############

rm -rf ${SQUASHFS_ROOT_DIR} ${PATH_TO_PREP}/filesystem.squashfs ${SWAP_FILE}

#########
#  END  #
#########
