#
# File: FindCppUnit.cmake
#
# Date: Fri Jan 03 12:00:00 EST 2017
# Author: gustavo.garcia
#
# Copyright 2012 Smartmatic Corp. All Rights Reserved.
#
# Confidential Information of Smartmatic Labs. Not for disclosure or distribution
# prior written consent. This software contains code, techniques and know-how which
# is confidential and proprietary to Smartmatic.
#
# Use of this software is subject to the terms of an end user license agreement.
#
# Try to find CppUnit library
# It sets the following variables:
#
#  CPPUNIT_FOUND       - Set to false, or undefined, if CppUnit isn't found.
#  CPPUNIT_INCLUDE_DIRS - The CppUnit include directory.
#  CPPUNIT_LIBRARIES     - The CppUnit library to link against.
#

INCLUDE (FindPackageHandleStandardArgs)

FIND_PATH(CPPUNIT_INCLUDE_DIRS cppunit/Test.h)
FIND_LIBRARY(CPPUNIT_LIBRARIES NAMES cppunit)


IF (CPPUNIT_INCLUDE_DIRS AND CPPUNIT_LIBRARIES)

   # show which CppUnit was found only if not quiet
   IF (NOT CppUnit_FIND_QUIETLY)
      MESSAGE(STATUS "Found CppUnit: ${CPPUNIT_LIBRARIES}")
   ENDIF (NOT CppUnit_FIND_QUIETLY)

ELSE (CPPUNIT_FOUND)

   # fatal error if CppUnit is required but not found
   IF (CppUnit_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "Could not find CppUnit")
   ENDIF (CppUnit_FIND_REQUIRED)

ENDIF ()

FIND_PACKAGE_HANDLE_STANDARD_ARGS (CppUnit REQUIRED_VARS CPPUNIT_LIBRARIES CPPUNIT_INCLUDE_DIRS)
MARK_AS_ADVANCED (CPPUNIT_LIBRARIES CPPUNIT_INCLUDE_DIRS)

