#
# File: FindLibXmlXX.cmake
#
# Author: manuel.arguelles
#
# Copyright 2017 Smartmatic Corp. All Rights Reserved.
#
# Confidential Information of Smartmatic Labs. Not for disclosure or distribution
# prior written consent. This software contains code, techniques and know-how which
# is confidential and proprietary to Smartmatic.
#
# Use of this software is subject to the terms of an end user license agreement.
#
# Tries to find libxml++ library.
#
# Once done this will define:
#
# LIBXMLXX_FOUND        - True if the library was found
# LIBXMLXX_INCLUDE_DIR  - Location of the header files
# LIBXMLXX_LIBRARIES    - List of the libraries to link with
#

INCLUDE (FindPackageHandleStandardArgs)

FIND_PACKAGE (PkgConfig QUIET)

PKG_CHECK_MODULES (PC_LIBXMLXX QUIET libxml++-2.6)

FIND_PATH (
  LIBXMLXX_INCLUDE_DIR1
  NAMES libxml++/libxml++.h
  HINTS ${PC_LIBXMLXX_INCLUDEDIR} ${PC_LIBXMLXX_INCLUDE_DIRS}
  )

FIND_PATH (
  LIBXMLXX_INCLUDE_DIR2
  NAMES libxml++config.h
  HINTS ${PC_LIBXMLXX_INCLUDEDIR} ${PC_LIBXMLXX_INCLUDE_DIRS}
  )

FIND_LIBRARY (
  LIBXMLXX_LIBRARIES
  NAMES xml++-2.6
  HINTS ${PC_LIBXMLXX_LIBDIR} ${PC_LIBXMLXX_LIBRARY_DIRS}
  )

IF (LIBXMLXX_INCLUDE_DIR1 AND LIBXMLXX_INCLUDE_DIR2)
  SET (LIBXMLXX_INCLUDE_DIR ${LIBXMLXX_INCLUDE_DIR1} ${LIBXMLXX_INCLUDE_DIR2})
ENDIF ()

FIND_PACKAGE_HANDLE_STANDARD_ARGS (
  LIBXMLXX
  REQUIRED_VARS LIBXMLXX_LIBRARIES LIBXMLXX_INCLUDE_DIR
  VERSION_VAR PC_LIBXMLXX_VERSION
  )

MARK_AS_ADVANCED (LIBXMLXX_LIBRARIES LIBXMLXX_INCLUDE_DIR)

