#
# File: FindXercesC.cmake
#
# Date: Mon Aug 26 12:00:00 EST 2012
# Author: angel.castillo
#
# Copyright 2012 Smartmatic Corp. All Rights Reserved.
#
# Confidential Information of Smartmatic Labs. Not for disclosure or distribution
# prior written consent. This software contains code, techniques and know-how which
# is confidential and proprietary to Smartmatic.
#
# Use of this software is subject to the terms of an end user license agreement.
#
# - Try to find Xerces-C
# Once done this will define
#
# XERCESC_FOUND   - system has Xerces-C
# XERCESC_INCLUDE - the Xerces-C include directory
# XERCESC_LIBRARY - Link these to use Xerces-C
# XERCESC_VERSION - Xerces-C found version
#

# Already in cache, be silent.
IF (XERCESC_INCLUDE AND XERCESC_LIBRARY)
  SET(XERCESC_FIND_QUIETLY TRUE)
ENDIF ()

OPTION (XERCESC_STATIC "Set to ON to link your project with static library (instead of DLL)." ON)

IF (NOT ${XERCESC_WAS_STATIC} STREQUAL ${XERCESC_STATIC})
  UNSET (XERCESC_LIBRARY CACHE)
  UNSET (XERCESC_LIBRARY_DEBUG CACHE)
ENDIF ()

SET (XERCESC_WAS_STATIC ${XERCESC_STATIC} CACHE INTERNAL "")

FIND_PATH (
  XERCESC_INCLUDE
  NAMES xercesc/util/XercesVersion.hpp
  PATHS
    $ENV{XERCESC_INCLUDE_DIR}
    ${XERCESC_INCLUDE_DIR}
    /usr/local/include
    /usr/include
)

IF (XERCESC_STATIC)
  FIND_LIBRARY (
    XERCESC_LIBRARY
    NAMES xerces-c_static_3 xerces-c-3.1 xerces-c
    PATHS
      $ENV{XERCESC_LIBRARY_DIR}
      ${XERCESC_LIBRARY_DIR}
      /usr/lib
      /usr/local/lib
  )

  FIND_LIBRARY (
    XERCESC_LIBRARY_DEBUG
    NAMES xerces-c_static_3D xerces-c-3.1D
    PATHS
      $ENV{XERCESC_LIBRARY_DIR}
      ${XERCESC_LIBRARY_DIR}
      /usr/lib
      /usr/local/lib
  )

  ADD_DEFINITIONS (-DXERCES_STATIC_LIBRARY)
ELSE ()
  FIND_LIBRARY (
    XERCESC_LIBRARY
    NAMES xerces-c_3
    PATHS
      $ENV{XERCESC_LIBRARY_DIR}
      ${XERCESC_LIBRARY_DIR}
  )

  FIND_LIBRARY (
    XERCESC_LIBRARY_DEBUG
    NAMES xerces-c_3D
    PATHS
      $ENV{XERCESC_LIBRARY_DIR}
      ${XERCESC_LIBRARY_DIR}
  )
ENDIF (XERCESC_STATIC)

IF (XERCESC_INCLUDE AND XERCESC_LIBRARY)
  SET (XERCESC_FOUND TRUE)
ELSE ()
  SET (XERCESC_FOUND FALSE)
ENDIF ()

IF (XERCESC_FOUND)

  FIND_PATH (XERCESC_XVERHPPPATH NAMES XercesVersion.hpp PATHS ${XERCESC_INCLUDE} PATH_SUFFIXES xercesc/util)

  IF (${XERCESC_XVERHPPPATH} STREQUAL XERCESC_XVERHPPPATH-NOTFOUND)
    SET (XERCES_VERSION "0")
  ELSE ()
    FILE (READ ${XERCESC_XVERHPPPATH}/XercesVersion.hpp XVERHPP)

    STRING (REGEX MATCHALL "\n *#define XERCES_VERSION_MAJOR +[0-9]+" XVERMAJ ${XVERHPP})
    STRING (REGEX MATCH "\n *#define XERCES_VERSION_MINOR +[0-9]+" XVERMIN ${XVERHPP})
    STRING (REGEX MATCH "\n *#define XERCES_VERSION_REVISION +[0-9]+" XVERREV ${XVERHPP})

    STRING (REGEX REPLACE "\n *#define XERCES_VERSION_MAJOR +" "" XVERMAJ ${XVERMAJ})
    STRING (REGEX REPLACE "\n *#define XERCES_VERSION_MINOR +" "" XVERMIN ${XVERMIN})
    STRING (REGEX REPLACE "\n *#define XERCES_VERSION_REVISION +" "" XVERREV ${XVERREV})

    SET (XERCESC_VERSION ${XVERMAJ}.${XVERMIN}.${XVERREV})

  ENDIF ()

  IF (NOT XERCESC_FIND_QUIETLY)
    MESSAGE (STATUS "Found XercesC: ${XERCESC_LIBRARY} (found version ${XERCESC_VERSION})")
  ENDIF ()
ELSE ()
  MESSAGE (FATAL_ERROR "Could not find XercesC!")
ENDIF ()

MARK_AS_ADVANCED (XERCESC_INCLUDE XERCESC_LIBRARY)