/*
 * BeforeGeneratingResultsComplianceVerificator.hxx
 *
 *  Created on: Feb 14, 2018
 *      Author: eduardo.fernandez
 */

#ifndef BEFORE_GENERATING_RESULTS_COMPLIANCE_VERIFICATOR_HXX
#define BEFORE_GENERATING_RESULTS_COMPLIANCE_VERIFICATOR_HXX

#include <string>
#include <list>
#include <vector>
#include <map>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            class ElectoralConfiguration;

            namespace Vote
            {
                class StationTallyWrapper;
            }

            namespace PBReports
            {
                class VoteResults;
            }
        }

        namespace Operation
        {
            namespace VoteCompliance
            {
                class VoteComplianceVerificationManager;

                /**
                 * @class	BeforeGeneratingResultsComplianceVerificator
                 * @brief	Header class BeforePrintingVoteComplianceVerificator.
                 * @author	eduardo.fernandez
                 * @date	Feb 14, 2018
                 */
                class BeforeGeneratingResultsComplianceVerificator
                {
                    public:

                        /**
                         * @fn	BeforeGeneratingResultsComplianceVerificator::BeforeGeneratingResultsComplianceVerificator ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                         * @brief	Class constructor.
                         * @param	electoralConfiguration	electoral configuration.
                         * @author	eduardo.fernandez
                         * @date	Feb 14, 2018
                         */
                        explicit BeforeGeneratingResultsComplianceVerificator(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration = NULL);

                        /**
                         * @fn	virtual BeforeGeneratingResultsComplianceVerificator::~BeforeGeneratingResultsComplianceVerificator();
                         * @brief	Finalizer.
                         * @author	eduardo.fernandez
                         * @date	Feb 14, 2018
                         */
                        virtual ~BeforeGeneratingResultsComplianceVerificator();


                        /**
                         * @fn  bool BeforeGeneratingResultsComplianceVerificator::checkTotalCumulativeSelections(const size_t totalVotes, const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);
                         * @brief    Checks whether the cumulative number of categorized votes don't exceed the total number of votes.
                         * @param    voteResults The vote results.
                         * @param    totalVotes The total number of votes.
                         * @return   Whether the cumulative number of categorized votes don't exceed the total number of votes.
                         * @author   eduardo.fernandez
                         * @date     Feb 14, 2018
                         */
                        bool checkTotalCumulativeSelections(
                            const size_t totalVotes,
                            const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);

                        /**
                         * @fn  bool BeforeGeneratingResultsComplianceVerificator::checkIndividualCumulativeSelections(const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);
                         * @brief    Checks whether the cumulative number of categorized votes for a single list don't
                         *           exceed the total number of votes for that list.
                         * @param    voteResults The vote results.
                         * @return   Whether the cumulative number of categorized votes for a single list don't
                         *           exceed the total number of votes for that list.
                         * @author   eduardo.fernandez
                         * @date     Feb 15, 2018
                         */
                        bool checkIndividualCumulativeSelections(const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);

                    private:

                        static Smartmatic::Log::ISMTTLog * m_logger;	///< the logger
                        VoteComplianceVerificationManager * m_voteComplianceVerificationManager;
                };
            }
        }
    }
}

#endif /* BEFORE_GENERATING_RESULTS_COMPLIANCE_VERIFICATOR_HXX */
