/**
 * @file	CustomRemarkDiagnoseWidget.cxx
 * @brief	bar code diagnosis widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "CustomRemarkDiagnoseWidget.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Configuration/s-m-t-t-g-u-i-conf-manager.h"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "../../../config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Configuration;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Operation;

Smartmatic::Log::ISMTTLog * CustomRemarkDiagnoseWidget::logger = Smartmatic::Log::SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.CustomRemarkDiagnoseWidget", GETTEXT_PACKAGE );

CustomRemarkDiagnoseWidget::CustomRemarkDiagnoseWidget ( bool showKeyBoard )
    : BasicDiagnosticWidget ( true, false )
{
    this->showKeyBoard = showKeyBoard;
    keyboard = NULL;

    alignment = new Gtk::Alignment ( 0.5, 0.5, 0.8, 0.6 );
    vbox = new SmartmaticBasicVBox ( false, 40 );
    remark = new SmartmaticTextView ();
    infoMessage = new SmartmaticBasicLabel();

    signalOkDetected.connect ( sigc::mem_fun ( *this, &CustomRemarkDiagnoseWidget::end ) );

    vbox->pack_start ( *infoMessage, true, false, 0 );
    vbox->pack_start ( *remark, true, true, 0 );

    if ( ( showKeyBoard ) || ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) ) )
    {
        keyboard = new Keyboard ();
        vbox->pack_end ( *keyboard, true, false, 0 );

        keyboard->signal_detected.connect ( sigc::mem_fun ( *this, &CustomRemarkDiagnoseWidget::onKeyPress ) );
        keyboard->signalDelete_detected.connect ( sigc::mem_fun ( *this, &CustomRemarkDiagnoseWidget::onKeyDeletePress ) );
        keyboard->signalEnter_detected.connect ( sigc::mem_fun ( *this, &CustomRemarkDiagnoseWidget::addEndLine ) );
        keyboard->signalUp_detected.connect ( sigc::mem_fun ( *this, &CustomRemarkDiagnoseWidget::moveUp ) );
        keyboard->signalDown_detected.connect ( sigc::mem_fun ( *this, &CustomRemarkDiagnoseWidget::moveDown ) );
        keyboard->signalLeft_detected.connect ( sigc::mem_fun ( *this, &CustomRemarkDiagnoseWidget::moveLeft ) );
        keyboard->signalRight_detected.connect ( sigc::mem_fun ( *this, &CustomRemarkDiagnoseWidget::moveRight ) );

        keyboard->setSensitiveButton ( kbEsc, false );
        keyboard->setSensitiveButton ( kbUp, false );
        keyboard->setSensitiveButton ( kbDown, false );
        keyboard->setSensitiveButton ( kbLeft, false );
        keyboard->setSensitiveButton ( kbRight, false );
    }

    int width = SAESGUIConfigurationManager::getInstance()->getCustomRemarkDiagnoseConfiguration().getTextBoxWidth();
    int height = SAESGUIConfigurationManager::getInstance()->getCurrentMachineModelConfiguration().getCustomRemarkDiagnoseTextBoxHeight();
    int max = SAESGUIConfigurationManager::getInstance()->getCustomRemarkDiagnoseConfiguration().getTextBoxMaxLength();

    remark->set_size_request ( width, height );
    remark->setMax ( max );

    LabelConfiguration infoConf = SAESGUIConfigurationManager::getInstance()->GetDiagnosisLabelConfiguration();
    LabelConfiguration::SetLabelConfiguration ( infoMessage, infoConf );

    alignment->add ( *vbox );
    setWidget ( alignment );
}

CustomRemarkDiagnoseWidget::~CustomRemarkDiagnoseWidget()
{
    remove();

    if ( keyboard )
    {
        delete keyboard;
        keyboard = NULL;
    }

    if ( infoMessage )
    {
        delete infoMessage;
        infoMessage = NULL;
    }

    if ( remark )
    {
        delete remark;
        remark = NULL;
    }

    if ( alignment )
    {
        delete alignment;
        alignment = NULL;
    }
}

void CustomRemarkDiagnoseWidget::initialize()
{
    showButtons ( true, false, false );
    setOkButtonSensitive ( true );

    infoMessage->set_text ( _ ( "Custom Remark Information" ) );
    remark->setText ( "" );
    remark->focus();
}

void CustomRemarkDiagnoseWidget::dispose()
{

}

void CustomRemarkDiagnoseWidget::onKeyPress ( std::string data )
{
    remark->insert ( data );
}

void CustomRemarkDiagnoseWidget::onKeyDeletePress()
{
    remark->deleted ();
}

void CustomRemarkDiagnoseWidget::addEndLine()
{
    remark->insert ( "\n" );
}

void CustomRemarkDiagnoseWidget::moveUp()
{
    remark->up();
}

void CustomRemarkDiagnoseWidget::moveDown()
{
    remark->down();
}


void CustomRemarkDiagnoseWidget::moveLeft()
{
    remark->left();
}

void CustomRemarkDiagnoseWidget::moveRight()
{
    remark->right();
}

void CustomRemarkDiagnoseWidget::end()
{
    unsigned int maxLines = SAESGUIConfigurationManager::getInstance()->getCustomRemarkDiagnoseConfiguration().getTextBoxMaxLines();
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setObservation ( remark->getText(), maxLines );
    emit();
}
