/**
 * @file PBEKeyDerivatorHMAC.cpp
 * @brief Body class PBE Key derivator hmac
 * @date  05/04/2011
 * @author Freddy Gomez
 */

#include "PBEKeyDerivatorHMAC.h"

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System;

PBEKeyDerivatorHMAC::PBEKeyDerivatorHMAC()
{

}

PBEKeyDerivatorHMAC::~PBEKeyDerivatorHMAC()
{

}

void PBEKeyDerivatorHMAC::SetSalt ( const SafeBuffer & salt )
{
    this->salt.Clear();
    this->salt.AddData ( salt );
}

void PBEKeyDerivatorHMAC::GenerateKey ( const SafeBuffer & passPhrase, SafeBuffer & outKey, MsgDigestUtil::DigestAlgType mdAlg, unsigned int keyLen, unsigned int iterations )
{
    outKey.Clear();

    if ( passPhrase.GetSize() == 0 )
    {
        return;
    }

    const EVP_MD * digest = DigestAlgorithmLookup::LookupDigest ( mdAlg );

    keyLen = keyLen > 0 ? keyLen : 32;

    unsigned char * ptr = new unsigned char [ keyLen ];

    ( void ) PKCS5_PBKDF2_HMAC ( ( const char * ) passPhrase.GetData(), passPhrase.GetSize(), salt.GetData(), salt.GetSize(), iterations, digest, keyLen, ptr );

    outKey.AddData ( ptr, keyLen );

    delete [] ptr;
}
