#ifndef TEST_CERTIFICATEVALIDATOR
#define TEST_CERTIFICATEVALIDATOR

#ifdef ECUT_MAIN

#include <System/Security/Cryptography/CertificateValidator.h>
#include <cppunit/extensions/HelperMacros.h>
#include <System/Encoding/Base64Encoder.h>
#include <System/Security/OpenSSLCertificate.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include <tests/StoreMockFile.hxx>
#include <System/IO/File.h>

using namespace Smartmatic::System;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::Security::Cryptography;

/*
 * Not Valid Before:	2013-12-13
 * Not Valid After:		1977-10-13
 */
static std::string validSelfSignedCert_PEM = "-----BEGIN CERTIFICATE-----\n"
        "MIICkzCCAfwCCQCH9y1/gOdMRTANBgkqhkiG9w0BAQUFADCBjTELMAkGA1UEBhMC\n"
        "QkUxDzANBgNVBAgMBlN0YXRlMTEOMAwGA1UEBwwFQ2l0eTExFjAUBgNVBAoMDU9y\n"
        "Z2FuaXphdGlvbjExFDASBgNVBAsMC0RlcGFydG1lbnQxMQ4wDAYDVQQDDAVOYW1l\n"
        "MTEfMB0GCSqGSIb3DQEJARYQZW1haWwxQGVtYWlsLmNvbTAeFw0xMzEyMTMxNTUy\n"
        "NTNaFw03NzEwMTMwOTI0MzdaMIGNMQswCQYDVQQGEwJCRTEPMA0GA1UECAwGU3Rh\n"
        "dGUxMQ4wDAYDVQQHDAVDaXR5MTEWMBQGA1UECgwNT3JnYW5pemF0aW9uMTEUMBIG\n"
        "A1UECwwLRGVwYXJ0bWVudDExDjAMBgNVBAMMBU5hbWUxMR8wHQYJKoZIhvcNAQkB\n"
        "FhBlbWFpbDFAZW1haWwuY29tMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCs\n"
        "MVz7QxionM8iGh1G+S0vdTcmrQy2HNWl13+ChtK7ErgdK5Ydd0EzF6p7/olAqGHQ\n"
        "WprrLZyxP+2DW/sbA2N0gaVl5kg4ap7K6IhucqIZmOUl0MS/YBRzC5wGxCjm1xaM\n"
        "sKE/FlPR1+WlpYJpZpFQFnFupT3rkhgxysGoIu9jawIDAQABMA0GCSqGSIb3DQEB\n"
        "BQUAA4GBAJ0y9XMR5IdZlM09WJyPGRXaRJnZ85NQ46e9IAHJpF3UbJywpUPSYfcU\n"
        "M70yj8V1B9FIZOqIcvXNeIniIF/QT1rHhuVAP6h663hDb9j3mqIw/dWHkjY9M3xe\n"
        "c7Sjx2pxNvpBdWd8kvHDL1j8oBak3b+1iCnbbpiwqIjO2TtUY1AT\n"
        "-----END CERTIFICATE-----";

/*
 * Not Valid Before:	2013-12-30
 * Not Valid After:		2112-07-24
 */
static std::string validCertified_PEM = "-----BEGIN CERTIFICATE-----\n"
                                        "MIIC9zCCAmCgAwIBAgICEAMwDQYJKoZIhvcNAQEEBQAweTELMAkGA1UEBhMCQkUx\n"
                                        "ETAPBgNVBAgTCEJydXNzZWxzMRIwEAYDVQQHEwlFdHRlcmJlZWsxEzARBgNVBAoT\n"
                                        "ClNtYXJ0bWF0aWMxHDAaBgNVBAsTE0VsZWN0b3JhbCBTb2x1dGlvbnMxEDAOBgNV\n"
                                        "BAMTB1JPT1QgQ0EwIBcNMTMxMjMwMTUxMjUzWhgPMjExMjA3MjQxNTEyNTNaMIGJ\n"
                                        "MQswCQYDVQQGEwJCRTERMA8GA1UECBMIQnJ1c3NlbHMxEzARBgNVBAoTClNtYXJ0\n"
                                        "bWF0aWMxHDAaBgNVBAsTE0VsZWN0b3JhbCBTb2x1dGlvbnMxDzANBgNVBAMTBlVz\n"
                                        "ZXIgMjEjMCEGCSqGSIb3DQEJARYUdXNlcjJAc21hcnRtYXRpYy5jb20wgZ8wDQYJ\n"
                                        "KoZIhvcNAQEBBQADgY0AMIGJAoGBAM2ew78JzhX3GocanVAxqiv0pPcKjUPhtFpD\n"
                                        "S2uI12SNIyNQqszPXF51euV5g8If308PVhM59Vpk+APR1vqfjrYvVuMJCBFI5IFu\n"
                                        "HFh0uhqArqL6LoKc2zs5xTbrl+ueyzZkCOLdmoolgaSzas+qbLGeO86ub5qXROTw\n"
                                        "xj6ijfmNAgMBAAGjezB5MAkGA1UdEwQCMAAwLAYJYIZIAYb4QgENBB8WHU9wZW5T\n"
                                        "U0wgR2VuZXJhdGVkIENlcnRpZmljYXRlMB0GA1UdDgQWBBTSCgQsgLAejEx7mR5g\n"
                                        "IRy6irmcBDAfBgNVHSMEGDAWgBSkK16P6n6Mm/Bx4DBAW81buzO8pzANBgkqhkiG\n"
                                        "9w0BAQQFAAOBgQCG6JPAyVrFc5/mTYIlY9/aIitOET7N14WoArAApwql7rwzPwgw\n"
                                        "slWSXI+SnVtiKPOSW75H01o0a1FjmA6jnc0fnoS/miCNPwPZCUw01VJK5+UTCz4+\n"
                                        "xjiljHFMKWIshq9FCYPoYzhAhW+dX9zsW+hUppJ8LLHIlTgXOPgP3NCFGA==\n"
                                        "-----END CERTIFICATE-----";

static std::string validCertified_DER = "MIIC9zCCAmCgAwIBAgICEAMwDQYJKoZIhvcNAQEEBQAweTELMAkGA1UEBhMCQkUx"
                                        "ETAPBgNVBAgTCEJydXNzZWxzMRIwEAYDVQQHEwlFdHRlcmJlZWsxEzARBgNVBAoT"
                                        "ClNtYXJ0bWF0aWMxHDAaBgNVBAsTE0VsZWN0b3JhbCBTb2x1dGlvbnMxEDAOBgNV"
                                        "BAMTB1JPT1QgQ0EwIBcNMTMxMjMwMTUxMjUzWhgPMjExMjA3MjQxNTEyNTNaMIGJ"
                                        "MQswCQYDVQQGEwJCRTERMA8GA1UECBMIQnJ1c3NlbHMxEzARBgNVBAoTClNtYXJ0"
                                        "bWF0aWMxHDAaBgNVBAsTE0VsZWN0b3JhbCBTb2x1dGlvbnMxDzANBgNVBAMTBlVz"
                                        "ZXIgMjEjMCEGCSqGSIb3DQEJARYUdXNlcjJAc21hcnRtYXRpYy5jb20wgZ8wDQYJ"
                                        "KoZIhvcNAQEBBQADgY0AMIGJAoGBAM2ew78JzhX3GocanVAxqiv0pPcKjUPhtFpD"
                                        "S2uI12SNIyNQqszPXF51euV5g8If308PVhM59Vpk+APR1vqfjrYvVuMJCBFI5IFu"
                                        "HFh0uhqArqL6LoKc2zs5xTbrl+ueyzZkCOLdmoolgaSzas+qbLGeO86ub5qXROTw"
                                        "xj6ijfmNAgMBAAGjezB5MAkGA1UdEwQCMAAwLAYJYIZIAYb4QgENBB8WHU9wZW5T"
                                        "U0wgR2VuZXJhdGVkIENlcnRpZmljYXRlMB0GA1UdDgQWBBTSCgQsgLAejEx7mR5g"
                                        "IRy6irmcBDAfBgNVHSMEGDAWgBSkK16P6n6Mm/Bx4DBAW81buzO8pzANBgkqhkiG"
                                        "9w0BAQQFAAOBgQCG6JPAyVrFc5/mTYIlY9/aIitOET7N14WoArAApwql7rwzPwgw"
                                        "slWSXI+SnVtiKPOSW75H01o0a1FjmA6jnc0fnoS/miCNPwPZCUw01VJK5+UTCz4+"
                                        "xjiljHFMKWIshq9FCYPoYzhAhW+dX9zsW+hUppJ8LLHIlTgXOPgP3NCFGA==";

static std::string validCertified_PKCS12 = "MIIKcQIBAzCCCjcGCSqGSIb3DQEHAaCCCigEggokMIIKIDCCBx8GCSqGSIb3DQEH"
        "BqCCBxAwggcMAgEAMIIHBQYJKoZIhvcNAQcBMBwGCiqGSIb3DQEMAQYwDgQI1DfW"
        "c2iuUWECAggAgIIG2Gtew4WNTrGALpGtMr+FcKaqpfreGmx3AmIlHXIMdb4GoPXr"
        "aGg4+ocpxetpGMw5TUkvCziQpXQ/8DPZFzzf0R4rWDKTdrb9Z7o8gD9tOsQD1JRT"
        "tnexdfL7aCTlOcDbaVP3u3ZpwDuTgKyLmO0ReXsOmTOBmv3v9aZQ88KxM9QcIf65"
        "Y3x+6usRE42OLWOxg2Tjm5DxrxLIB/nAZCSkRcnOEBgBQHr1BShNyqqn81ESmYlj"
        "/XjTlaRMtCFsAiJKVuzke9sVaISrqNN038jMFlRlH3BRabdYwrJXu6uyOfPUl9cS"
        "FE/cZ5ivANwierLwMDRC/UDQkSSdIqc3YBtbUr03xxpD3bIYWDdzdBqaNGVjXto8"
        "ZpTBknyb4GKqHqGLvN7y6xRS48JR/Z4VlA52vGTsJQy+gwK/ti5nhGY2ex0hmrLe"
        "hC7vi3s+WUkUBII9UQpebRTaVnagS0kC8/DJCn8R0W2/07m5LXFhVrDw5GCFF6xY"
        "b+59LGhVK/RnIDgOevLg4eTAAmGmthlTm1TxVtsBNblXjNbLLte93Eb83Qr3Kvzy"
        "Q/2KJcEUkCEnRaYHYcN3ItZUMRcDMOPGe3BzsM4A7qDfJCNOn8mWghS4fV3Wfqjg"
        "d0TYrj2wMvoooG9v2n6z4nFmbEhXVq5AQPdeb/vQWdZabzOSyDf2oUmU8IpDsnCj"
        "SgOt1Uuh31CyNM2nY3mWUEkAqOYeL4QcLVeSzxkubEA6OKeMCK69fpjvW0/YdCyA"
        "aliuhV9N3iiAQsFv8eje+GK9AZtf6DHoUaJtlIrsIKvrIk4SFjPKz6AkwUKBvp2c"
        "V0lGxqdSmgiJ8JMfmgazQV6gi6ZqiYVl1B4dHjXZ/YornJmWgtZiVRQcDq2tRX8x"
        "FEW6oYEG/Emy8IieeaBxQ88XYL1xEW0UUsRe1mtw5FaiH5WHCG1jUaVMaed7LOC1"
        "ap0O7arxIqz7DFQE7e+wBxGisTpvutM0Xs8DXI6R63fBepBBidLWTeFQSTLEFW01"
        "JKZ+fJUwASA5q6wgdl/bf1yrFzl1pld+JEWzkd2eCu9ubezdVn2OhY9MlwB64aCx"
        "PooOp9eZmt+bYD38fIVCjDomsqLvoHm6T6bCHj4Y8I/nl67wX99q36U6c0WHGKpY"
        "71o4DW77ejtiVA59K4j8QklVZpTM+FmjuyrTUHiOF8Lb37GZL8hdI/zCHoUrIG15"
        "Xvr/9Bm3nkjPc6IY1i9qwHeUtXaxFDFXzAjzqcaEvIx56O6EvAuyrT6rcdnVV98P"
        "BvCcxW65fxW9Yq7KsbpB/DasF1TD8ENWowW914qfblyVglYGi8sxo5M8atTxmL2k"
        "lGwsL9iEJzwbXBmp12dHdjkKKwMI01HW73tQsDtVeE76gWe0WLObsxfQIDG2Bol8"
        "zc+SW356ayEOQ04QZNQQ0A3mCVtNBcEKZpHdY+JxNcb+j47nxmEvbB00M/KR6QXd"
        "DcGhOPQSDLvrGl9iC79hQuWBUoxVuAX2J8adKlKl5i+elBbw4Ow5oDpirdQERdZW"
        "oqk6ioiJbvvf6FvdLiQnYS/tTEiO/+6dz+Lpu6Nv2bG9i6bgWHWk++PEWRlUutKl"
        "sGItFpmQUYZ5/sH1bExedgLfvFIJ+UKBrOLNLgmTmIvFoxoMcRg5zacJhyOOvuZC"
        "4Tv19spKku2cHEeLXNl42ZQWOwoBQvVtKcFWPxm4ZY9l+j6Z4QdlRDoePRt6CyGO"
        "3C2qexsJ6y2bffFkIxiEc8/I48FSo4ilVNrS3hVIT8pwLZbCviBmTug4DsS617kb"
        "UHp4vznGosCi+ZrOwR/rL+5WtRrXbCWnqwrVImAClF8fCehJ6QAxOKrOioKXT0C5"
        "OKxWXmOR9eADSG+wPD88DBDxSgLkZtlsWBTkSoW3E5+ZxI2i/JuKrPohYtWEyUK5"
        "5/fBOA1nuiCHQjw90tNwtynFQzOtDZtMO2F062c3iun7bCP+0f82SfGTI/y1r9Bm"
        "XwoT1NKMY5cUlnlg69jy0prxYi+92/A6uSYMNV8V+gikGp+ItTzmWqIktxGz54Yt"
        "nQDtg0ncqko59pjf294rFAaHAS8iug4vt2ZwoG8nbK0Jq1GJuscsuJhWG69Fh2XO"
        "OWAQYfRa+IIXiC61woRytU9kKrmKAdW8xuWONtdjNKOh88wvvYLypPB7H/5ueD8c"
        "6qfuJy07YDXhlf+WTv9CpsUGKTBeBPTgOj+LGig70SZopOTeyL6m6fjkFFK2DZD6"
        "MKPrAu0CVDzA3+o0QPdZox1rfd0oYcIl+5oKA9nYl1rG13pullDdy93KNGZZhiky"
        "dKOtrGt7WTY7o8/BRNxAzAGlI8t77A22KqSHAAXCy0rkkTwRGjCCAvkGCSqGSIb3"
        "DQEHAaCCAuoEggLmMIIC4jCCAt4GCyqGSIb3DQEMCgECoIICpjCCAqIwHAYKKoZI"
        "hvcNAQwBAzAOBAh/liYliUlRvwICCAAEggKAZDhx2m31ZDk0xsQGXbS7qrx7DFkM"
        "xbbgK9agDLJ0zQ3StaGo05DLdqmD6y3erCgrT4FxLxwvjxr9q08bSOu5ZvANYlBC"
        "p8GUcRhDmswhkGPBmLh7bbVU6mFck02pr1kntm1qFr05JYZ+DFIYS+hwoPv0DU8m"
        "C7VCYV/crUHrfjBow5i8XKngd6q9UwxbrkLlDhRgmHwlAlWXg767C+Ds8TYe5wgr"
        "ln5wft7R757vITytvQC0Ul3ELdAj+7jyiifsuLwG/BgJT82BU6M5NtMG9pJKafpj"
        "3RSIqqfrDz9TppG4AcoisFjUj67ZES2Y1uUEhW7V1M8elbVl1zsv9GQFr90IP82o"
        "rb+UVpmUoR49f25Eqw9HO48cHvqnjHwFDc+0AQiDmQNQHanp+H98hmQA0Z6qvaEE"
        "fKT8ITn8daUC7yP7zLefAauTnLlOw12G6YUIHD0CKt3uR3+OwifkHd2nVRKtjBqG"
        "7oXZiX5xaJ159ieiKZ+wXQGWXjyIIowHWhOnAuzMKV5uEm44KPKRWyQDNceR66mq"
        "tPPLiYXHX7WIJJh/ubazZ0Lh9zp5wTAefcDHhOjAlJr4K6Blb3fBOhtlqMQNK3A+"
        "IJqHJgcEvL2HN4n4CysVFrw+YG76snA2V1GttwUl95wpw1sgne5p5MR6s8XtYWf3"
        "Vbx6tELh8ivEWhqy0qHV+Yq2P6WmpTX0QBSieWJ4uCeTcEWiko1jvdTAIb1W1M8N"
        "oPStPiOBHFQ2r/wzo7co/30Z7e1F2ZSqMWfzXGx9cXzEu4RipIjiYTRxl0galgXM"
        "T42xlJp8Ut/36k0DYFU12Uzkj3Mw1Z89d5GbpmyIDmaSitIJ4w+2gM7cFzElMCMG"
        "CSqGSIb3DQEJFTEWBBQJ8cvFVKweYsoJq7FTQU2QoxBKaTAxMCEwCQYFKw4DAhoF"
        "AAQULiJWJiOVenea6+yEhex0GcuehmgECHH2ajsukIsvAgIIAA==";

/*
 * Not Valid Before:	2013-12-30
 * Not Valid After:		2112-07-24
 */
static std::string validCertifiedWithIntermediateCA_PEM = "-----BEGIN CERTIFICATE-----\n"
        "MIIDmTCCAwKgAwIBAgICEAEwDQYJKoZIhvcNAQEEBQAwfjELMAkGA1UEBhMCQkUx\n"
        "ETAPBgNVBAgTCEJydXNzZWxzMRIwEAYDVQQHEwlFdHRlcmJlZWsxEzARBgNVBAoT\n"
        "ClNtYXJ0bWF0aWMxHDAaBgNVBAsTE0VsZWN0b3JhbCBTb2x1dGlvbnMxFTATBgNV\n"
        "BAMTDFNJR05JTkcgQ0EgMTAgFw0xMzEyMzAxNTA4MzhaGA8yMTEyMDcyNDE1MDgz\n"
        "OFowgZ0xCzAJBgNVBAYTAkJFMREwDwYDVQQIEwhCcnVzc2VsczESMBAGA1UEBxMJ\n"
        "RXR0ZXJiZWVrMRMwEQYDVQQKEwpTbWFydG1hdGljMRwwGgYDVQQLExNFbGVjdG9y\n"
        "YWwgU29sdXRpb25zMQ8wDQYDVQQDEwZVc2VyIDExIzAhBgkqhkiG9w0BCQEWFHVz\n"
        "ZXIxQHNtYXJ0bWF0aWMuY29tMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDL\n"
        "6mOwfzpmEYrLmOxBb2q1q7OiTRA8aS1vn51O4F3WGJ0Yf25c55DK3QbHsuDtMiWz\n"
        "G1OHq8xYNts02+6EJEitgeFGMYRL2Vfp1t8GFim18dUPxdlSM9yWvEyzBOMtA73z\n"
        "v9t9gEEO7MGTvZKepOuE8fkk2aOWefS0vhfoGSbmqQIDAQABo4IBAjCB/zAJBgNV\n"
        "HRMEAjAAMCwGCWCGSAGG+EIBDQQfFh1PcGVuU1NMIEdlbmVyYXRlZCBDZXJ0aWZp\n"
        "Y2F0ZTAdBgNVHQ4EFgQUqoAa6O6yNd2vo9pVdBB8C0xtyZIwgaQGA1UdIwSBnDCB\n"
        "mYAU0M4d9ks4kIc5yQoladKsvQQrmzOhfaR7MHkxCzAJBgNVBAYTAkJFMREwDwYD\n"
        "VQQIEwhCcnVzc2VsczESMBAGA1UEBxMJRXR0ZXJiZWVrMRMwEQYDVQQKEwpTbWFy\n"
        "dG1hdGljMRwwGgYDVQQLExNFbGVjdG9yYWwgU29sdXRpb25zMRAwDgYDVQQDEwdS\n"
        "T09UIENBggIQAjANBgkqhkiG9w0BAQQFAAOBgQCwV43852OcjBc/pmj9PrgtvLdN\n"
        "rS14I2Wa1DZbgkj6r9/oADcRkvqZplgxabetkZgsSdRXMwzBUZkS/o52/puEqL3j\n"
        "PIKq3IyS7Nu5g6Fg0n5cnT8yZ+KyZTjj57ecOJdbxF5rVWlamscAlHz1qaY59wZl\n"
        "FdAfcgPCnBN5Zsyiww==\n"
        "-----END CERTIFICATE-----";

static std::string validCertifiedWithIntermediateCA_PKCS12 = "MIILEQIBAzCCCtcGCSqGSIb3DQEHAaCCCsgEggrEMIIKwDCCB78GCSqGSIb3DQEH"
        "BqCCB7AwggesAgEAMIIHpQYJKoZIhvcNAQcBMBwGCiqGSIb3DQEMAQYwDgQILMip"
        "vKjxNhACAggAgIIHeBGTreHijloBYyY5xhcO4UgR+5pzBPBU1IrxxqXgKwyGkDOe"
        "DoAbjfRoYuY0bLBV4GUK6eqnmtJbJEKNpkchc1DyHRINX7n4RfBmkzHRUeOj13Um"
        "TishDAxcLpCESLnVsWumf0fCFY30a0Ms+OLlnLFnLIcjt2R7Ex+O2+KhXXXFdesG"
        "6QuqSreTYCgT7ose74X//aqtLVliIDJnv9Lz9oC25vHtTyE2C0P4wkK/8cIZz7Xy"
        "YmmarSE18C1cvQCaR7BKwTOJuXcR7Q9neEgQ0UfDZjtUAGlX/wiztR2LVG9QXWy0"
        "S6krXb7nlhxKHFJ87S+OXfMi17XteIhu3clQiBA8sNLG4FoFWu+tWjZEydJW00hq"
        "bFToGY6b5ormjDGpZTdFxTyC1iVOhtUyuZSPNSV8I5NEXmRJnljPCwbTbwmdpNnL"
        "QRIwVFZt/L9BZRp7XYRJwSqdNLdDkPEq3WCxeVbl6uT6EpAffu4/d4/PbbWUDbdT"
        "i7tlXW68fdXwT010VXx7xJNDxI4+KppqhTEj+DE44MKrQFqFHFt/S9KUU3AT5Kog"
        "LFK6zbQSGFKQ3YFAI6JSRzKRMQAr5d7SUCLWlGTbOpRhTRLPpTVM3RXRWOQWIzHt"
        "8y8xq/Z0nOvNd28mNqEi+mwFLiunXwCUZhvfZWVJU1wr+iGVidMcpKnp1tHwAe1H"
        "6lFKbGkdBkBZQ0qCxlfuctCtPDeYiqib1R29caV+vsuVry0JSuTUwskMmrmWK6tz"
        "DLaoYhsIEHzj4zjfu277G2t2ceP3kGH4kbLhuAV4AzDVZGpWWOoBYS03XmuJdVTj"
        "bAYikL2sM8jJhiw71R8OZQU7KkUqM9vupw8HGzvn5d1Udy7k7BKSwcqw//18QSBb"
        "eIQbJP515M5ywYU7X+QAhg6uqzQ0Xhsk4tS3eITG901FLVkqePINVhGMPAfJZpY7"
        "FrI+LLcQ5K4Q1Mw2rVKwsd4qe3FF83JZ+W17Ut7DZBWUx4zKb0lv/es4ugZODbGB"
        "W0MeaB976InrC/VJjyEBs2fDK4n02cXlJXCkFsB7ASnZhRt9wLm46EuCM296g7Zq"
        "J5eXoaGzPHOXy9KgDgtJih1AAIYHQxl6T7/ZK7BrCwLVgfZW6pf+ndrrtqhPqWfc"
        "K75GHAJ3MNcBIQe7Xr1Z77TS4IjPZpvy7zBADxmbiCYqOUpDkQ/ptGMGdlw3K4pt"
        "SRBW8wdSUjsD6Qsx9N6LfTfruYjQxiJnpyGWJT2sPAnMy19Hsjply8jRqC5RCh1L"
        "00k+n8U+XSnyrV//xkO3Wxbv5NQkD40mCd4/RZ2vx9CeA2M6C2cQ7xS3/zsWmuOG"
        "xap5Z54JnjXcCq+cmbmQNrKhaJ2L2sFE6+FHA7dni/0ZYB38SPme0/DTjIWhyz6t"
        "/LZ3n3iyoZLdEMy2j2K6VMjQIJ3CxY/F2r1HtGFYWKs+aZNQYhWRjROIz/+AxS4q"
        "j8r/QZDfo8Qa50MwbBnAJkB757tJxNhym9pGETbXJ/TJcnmpSx2Ze+dN/knisWIf"
        "IY7w31akW6Ls53mYzld1gmLIyUmvz9QuzACkvK72YooKnq8IFmcXRAva5CToRNxo"
        "i+6n8EaD05xj91yoRDxYp1OJKk0IvgG8jgRN6MRjjxWw1J1zOQtv05aBhopIZD0/"
        "7z9hFi42/C9fd4kn8o+vKBPP+nOw4s72b3oXCWD+sjVH8pYvmz8OJ8Q4dzcpqQes"
        "soDSvq3rcOEOVxqgOJGdVXaIzIhUrfc/v9Mpsh2/fzLJPycd2+JEHb8lsQbdWEJ2"
        "r2hjj9TNaY+/FvF3vAIP/25iyzXZkzFR2eqyPYKEe1khT0ZRWMMk2wNJhrdc+QI5"
        "aiVY40v1yvVVCD4VofMbLwrqLnMYo0cg3vCG5IG7pwLVcvqMCCU/XZSVQT22/ZhP"
        "U+xyaF1a4ixuvXvfzDK50QxXmJuocgpk5j13z7Vh9zvL1LEpIPsUwKHXpDAt0ME/"
        "6/EgXDjTUrtOhX43CYy+ST9vzkJTKeTU/OwfnzhN8M0H5amZ5EjZWzEeICnnsSHL"
        "l7FgIIHm3mkoLJV40toG8Jr3N6kCGcVqUDhWpaHlDTRyZRJe7rO9OV2QRnN/u/rL"
        "6as+JyXB4a4G0wqFgqUANla/BEjWTty5eS8GhmXONAkhwBgf26dCl6mdmZ0P/JpA"
        "7iDB4lwB3+BUOO8xlzUUxqJ4HFkZ4M0nlC2cejxdLkjSBWNeTHaNq8onD4IYINAb"
        "RdFNsa8r7aH8pnj99MGHP6TSiojYS6krrad5u4IA9yGLiLo6LhPxqHOQU6RWkN0g"
        "6f7ScH+a4j07TpA9m76rZnTImTeQ1u3pDdyJx7msM7NbjumhUGnS5TS3GDfqASCA"
        "TZBRhRVtEDHwJp7hBbEpi+kUFRq0M3Y7Olcvd8OIgzWAG0Oxjod2CEdh9yhcPGQA"
        "aN+o8MS8FuG6Z9TAEcvGCApIm6XlT6DZRHXdKQHaNoPA7aR46k6RUAdGupZ4At6m"
        "D8di1MEHpn9ZUHYT34ETSJD1NRgsRalNtr71bQsz2XZ0Eoj9+r+2I5krAU/wb23A"
        "1voojI0wggL5BgkqhkiG9w0BBwGgggLqBIIC5jCCAuIwggLeBgsqhkiG9w0BDAoB"
        "AqCCAqYwggKiMBwGCiqGSIb3DQEMAQMwDgQIO3ja4yqBTXgCAggABIICgKMMM8K2"
        "Cg2Re3rFfcBFUf3APPSUjj/GWKzvNHOCukMTtym38+xKdiAU6KwkTNY9/AZq/2d8"
        "yY0EQSREPeTCg4Z1KeDqDaV83aYNZFomQBmQgk9zY0U7W9bsOaTVuK9SDagW1JQc"
        "UEGgF55ELgv3lrIXB4vRQfs6BewCPnzCYUvv0kn5BPtzw9e4fh7Sy189MYdmB0uX"
        "KeDkRShG5E486ZlliRuL/JlYe1hKmJT5hpzdUpQBz9z6KUL9CyOScu9vW6pjqRji"
        "8vhRG7aOoZvyfULbtce/3CBiXoVaf9f1WYW2/JmSMdMkRqDhFfbzUeD5yQwPLJqj"
        "NJt69yUgLmleb2PAsUk+/rKw/VIX2F/RV3YGcKGk0qlcbjsUNsPJQ9OoWgUqu/tu"
        "P9zlR1v703Dp1sQeoWLwtL0sjsgtUNR95bYTKxOxWOT66wKnt5qNQe5vzGy0EmyO"
        "SFIeC/8bJQ6zg17E198r/4Ci//tgH/Czy7ntvYSJl+9wLehf7U2GN+VWMl3hEcEv"
        "BOa7jDMVNiSYA2U8BvMXwqBsZddPtju2ygV0HAKG0F7jZczq4kh/Ma7kXciB8zL/"
        "GmzzRSopPBJw4uAzy4QYKMElAh5n1v0T1ib04+618V5IQe/wLgu15EVIl+1PUvkg"
        "lScmGLg9s+LpotcxVxd30embcMcnHJ4ue5/zABQKBKPssvJQjcaYop69op+VRqU5"
        "uZyozWf/7Tnlk2nc5rQNRisf774SNyu0mGU8Ro0eRYIoOUJdX17h6SPHbx6QnsKf"
        "ADUaxwktk76wWehAuv6i9xcrGc6t57rIkTZZghCs43incag5+64wzHzasPOoUbqi"
        "ewEX6bogzVQ3knMxJTAjBgkqhkiG9w0BCRUxFgQUmAfrzuaxctAKQ865xtkaJnHF"
        "STYwMTAhMAkGBSsOAwIaBQAEFEUsUKLrBKKRNC/54ClLPulgAYkpBAiwclqKnOBt"
        "LQICCAA=";

/*
 * Not Valid Before:	2013-12-30
 * Not Valid After:		2113-12-06
 */
static std::string validIntermediateCA_PEM = "-----BEGIN CERTIFICATE-----\n"
        "MIIDTzCCArigAwIBAgICEAIwDQYJKoZIhvcNAQEEBQAweTELMAkGA1UEBhMCQkUx\n"
        "ETAPBgNVBAgTCEJydXNzZWxzMRIwEAYDVQQHEwlFdHRlcmJlZWsxEzARBgNVBAoT\n"
        "ClNtYXJ0bWF0aWMxHDAaBgNVBAsTE0VsZWN0b3JhbCBTb2x1dGlvbnMxEDAOBgNV\n"
        "BAMTB1JPT1QgQ0EwIBcNMTMxMjMwMTUwNjA0WhgPMjExMzEyMDYxNTA2MDRaMH4x\n"
        "CzAJBgNVBAYTAkJFMREwDwYDVQQIEwhCcnVzc2VsczESMBAGA1UEBxMJRXR0ZXJi\n"
        "ZWVrMRMwEQYDVQQKEwpTbWFydG1hdGljMRwwGgYDVQQLExNFbGVjdG9yYWwgU29s\n"
        "dXRpb25zMRUwEwYDVQQDEwxTSUdOSU5HIENBIDEwgZ8wDQYJKoZIhvcNAQEBBQAD\n"
        "gY0AMIGJAoGBALup6IHsHpukWork6ZWVFsScifl7bdcBODG3NGQK+TGEk5e3ac/p\n"
        "uPc7AnQA1+pobOh9AAo3KzZ+bDV8W9ps/aEUiqvs4S1qvZ8dOxEslZZ27DH7xCpe\n"
        "8OT+jrjKcy9BcURzPQzlS//WQWSU99TukrL33G3VYK0q2wOujuzDs48VAgMBAAGj\n"
        "gd4wgdswHQYDVR0OBBYEFNDOHfZLOJCHOckKJWnSrL0EK5szMIGrBgNVHSMEgaMw\n"
        "gaCAFKQrXo/qfoyb8HHgMEBbzVu7M7ynoX2kezB5MQswCQYDVQQGEwJCRTERMA8G\n"
        "A1UECBMIQnJ1c3NlbHMxEjAQBgNVBAcTCUV0dGVyYmVlazETMBEGA1UEChMKU21h\n"
        "cnRtYXRpYzEcMBoGA1UECxMTRWxlY3RvcmFsIFNvbHV0aW9uczEQMA4GA1UEAxMH\n"
        "Uk9PVCBDQYIJAMjw2rh5O9/rMAwGA1UdEwQFMAMBAf8wDQYJKoZIhvcNAQEEBQAD\n"
        "gYEAJM4PDJn3+Y1m6kjM2X/M7UIG1RWkanMngtqEc4NGLEmlW/0X1XIUPqRStber\n"
        "FI6mzlxdYh40T7BRuCQAqSim02Xo0oRKiTppQb7vJeQdbto7G70E/Vme95mRLiSQ\n"
        "wUS3uVyl9M1NhnIxDtiRbc45pIYHUpSRQpBcFRPqSN0XEqI=\n"
        "-----END CERTIFICATE-----";

static std::string validIntermediateCA_PKCS12 = "MIIKyQIBAzCCCo8GCSqGSIb3DQEHAaCCCoAEggp8MIIKeDCCB3cGCSqGSIb3DQEH"
        "BqCCB2gwggdkAgEAMIIHXQYJKoZIhvcNAQcBMBwGCiqGSIb3DQEMAQYwDgQIk64v"
        "2Xd7rZMCAggAgIIHMPmxk4XWQ53r5kQbZhkFr2pI/q4VIjXwCQ40B+tJ8NLGa73i"
        "CTaeDNvb1pt+xiqXLuhMkFYzPE2pZSi9w5Y4rH756cSipymtZ+eKiOM7CFbRJqEs"
        "CXTDS4bqDaKDiOW8vVnP3rpnSI6vaBSNncyj7bgbH/5fV5jRvkzBoJjTxVt0tzmG"
        "k+zmmaWC9txbn6sRUr5660f7unSF1ra5bQJ5xBkcabArbk4nBs2ogdN90t666ma0"
        "qwpCa09Qt/gBpZaEHfFSkUhTuMoeg91VHiOISaxJKnc+FL8LF+Kukuews13CtT/8"
        "CjPihatrXfH6jNqLks4/c5Ol1e+el2LFlr6ghMOJ8/Bg1pScmQWQ3UpMR4vf2RyT"
        "cPd1GKRUAdjldbsrS9zKsUIMpM9FYGjnLJ+dg8Zp63ciw/wJcJ6VL6XXxepo0mu5"
        "aHZF9bIgiS2aI9bsgAmcZfkgNJFEPzBhOnt0X2xtKwbNAPQUb/4BaPdTg4WXUR4S"
        "+KSTrJ3xJTEvP7I+EXEMdtQreieFhBRrifoFY8ef8C5iN4VKHfe3IqqmpUed8JjH"
        "12dDtbas46Jvjcfg5Ek1/7p9+yP/x5HxD1EJEe7lK4iws2ncQego2WYI0l6lNGkN"
        "TQtkBO8fgU2s5dMHZfRm6ZZk7opd7+wqxpkwyFU0YiJLHi6+IfmdT6j7zUvQMQwS"
        "PtcpiD370M4iosbPVgVhmNqYq+aRO5EI7C9vlg6JHhBWBNtlZOheZsNny00DqTZd"
        "pe+lK9SIUT3lnc3IrUkF9hw1zzhnu76yAsTo/fyzCgXVX7pDnUQBNAqZHkBE3Xhp"
        "ZyN+ZLeBOa0L+KZ8uFN0uKvfL+6VQ2bADqs24KAVBsABQnWSsk1DjuHOpezJMDN2"
        "UmaXZYhD60nndY8RKpR13jSgXulH0ju6WOMJIzb3Q5qSjS8NQ9YAMGRJMgrY+y7H"
        "L6sZzRU6RYrpDK4AjI0QYaDKtcRWyrcbctaAraVyWb7mHCLBlMxaLjyGXqWWVAn6"
        "cDpzmWTPcZE0LJSmLH7B7qS9XhZ0i3Pxowg7pQrNlKGruHSqrsBrtZHHd/DwrB6E"
        "+D5U0grqCJZ41wc54zvHTT+WwnnCS52woXKHXNKZqu8tGGVrr2A82cMKhllZ1qzz"
        "PYT1Gbl0MB0UeZ6uM2nTDIGtCiS5WgoGSURKB+QjjNSuz7/4e0ytgsb9i6KyzW5q"
        "hhRfU7h5c9MZVocIhDxqy7ivQ3xSkY1UHdkliHoYG3M4SHk6dT84cJL3SC1838qR"
        "55CL4DVOxM4Ow1IWPMBc4YwUve/cOKJWQd66egRSlBdJvcOqNSbdqN7MBETbwxTb"
        "dnKG6VAX5NLc0Gnv2NCss0GVWdNsBFH7mYx1Me6CdQ/hDbn9rM23PwJfDhoMFtef"
        "SXhEfcc+Ot5KhNcS0CP6txzWjNXhbJn6hinWzBhIkoUrZ/FVOdosUxiq5wE8NhjZ"
        "/aq1J86Iw8IZEoxhOG7QqUyEQGXh8+cf3jIn22TcUWgRlosPtt0g2SSZEAgAMIll"
        "IP0EumqV1YM7YrK1fHk8KgDGgYSz0AsW/Ky6tYryEWyoJovf91XFB5jjO8WojB5D"
        "/azIOnB01X76eOxGdnUA9K1kxWOgTjTsZZYKoZ7Zs+UNpBXHT/0svAOR+uSBuMZj"
        "2n1LbgTwAuHCZ9JgRNn/Z+UUQzwDPy7XmcU/YT69mu3VxXhWiFvfwdlHVCaJ/sKZ"
        "2XI+WdjBWx0XZSl3b+I5LVm6GY85K3ovQizIDj1jcAm9MiSfEUYk5m7270ViUteG"
        "SLJ5cy+YCibPLcISxUgDxWnqrciBOkdWBvIme1BtOtjYMRXts1144tE3oHe6T9Qa"
        "nhIMM59RKGz1V6SS8eGjj0DoQhFsafRz5gZXykUKuKbJFULCO0MkXPK1i1dY4TxU"
        "KNqTm5txrf5bzaAMJ1g/UblHQdAdBBQTDePo7mEvB+KZh8yTWcgAwNRNOdmDUSJS"
        "MOFd+UjdDzqdh8mTApUgDYuydqfrlsf0v5/aQjsKU61qvF5iUiFBNm130rkwRT5F"
        "1T16tcPhP7AqIrPczgzLMMNnnmMGlPA2iEgDIzV+cZ6rFe2FVYcKDMeERefRpDCb"
        "WuVZuXwFsxXUNdHgIAPRBCTrtG1WMaSfJwbKR3R6f7Vhjv7kEedBa6p/8oJcHtc8"
        "Y0bM9Jyix3iWHIlttItQ2unJBS2J2V4KvQRxwv0D3fGX+ccsouQtpR0iUQfRRF8s"
        "hpD86DeAQaRTWlZzu0EoJO3b+JtyoYw+rUOKYDALMVSpH/rzeCAFHfUqBEH0ZTam"
        "1XKOMXFqAhbCzAWMalibH/xDbdZNnkCGlAhg7QNZXq8m3StQ9WFTXIbNBf8JT47y"
        "USI6H55h6j44NFkfM2O1Ma1tMR93o+ahzVzQDcuzfcZZVfEccXqen+JLgm26aVEC"
        "VGicrbSIH2dGhPi706zdGwXE0osYubhmSq5cTOswggL5BgkqhkiG9w0BBwGgggLq"
        "BIIC5jCCAuIwggLeBgsqhkiG9w0BDAoBAqCCAqYwggKiMBwGCiqGSIb3DQEMAQMw"
        "DgQI2ULOnkRvK3UCAggABIICgENJulTmxOOP0s8/HqkCKp7KiwzPH+UcJyH8sr/5"
        "m6/uGn8XMG1uKpre8aIigCk+hwh6cg4SbipUqG6bBcqbSgLtmcmnYEn7UimuRMcR"
        "spHlq0ot886bPSsaQzpxsmOsEBObogMwzEZ9nJPjBmnit7KGoZQj/tQg3xFPlxlY"
        "xE79FPY7CSzKfQjXXWaebQgiVyIbj4UQnjQ2r4IBiGmQIz/fdjz9PABVB/L5i9sX"
        "lte9bT21B/9jf9LsRWMLULAk+3vv1LaPL5vHA9E+HOVuSktfJ7FoOjInY9TXEoDD"
        "tE7jzDgaDJKDl45ip7CjPDsexarBql1/BXLjF4zp/u0r/qODDrpQniFBj/4fWNtz"
        "vffy4tnWPAFjjwzW/cbcAQWlfIEwJa0f6dIZxNKNV7jrxpuOSTLO97Ti9Gim5SIW"
        "ermuTdQv+dwFOfeotJoz61ZVOB1TL9p1JxdLpwSfG4Pd7jiUEGzIz/2MdONLyEn3"
        "HfXa30bsWofil6VnNzSUXOO8oAG8VacwG0dXDXnS+d6TNI+PdMj0zZ+pg5gYXIll"
        "pcw15O08nAGEGHY1KhTdZcNdhwVJUTV6zVj9fTCj3xEtStkW8TWyIIQUDIxCnr7y"
        "crTuttmsFMiSbwM7o3icL3GAcV33G6nstNsn3dDg2AEZTm8cgFwg/4xK8FlC6Mwq"
        "PV77u4iJ8h5Dtt0xPNDtcXXbgwdqMKYr0rThiHkBq6Tq/O43Ic54Ts1+bSBxNdc/"
        "ae0ukk5nQweyvikycqnwHAI5cACCI97WqzlvF6aTf1Wy9vPrwgIgqJevRq+LoUSl"
        "zG/01L4YvY5bkZCQFRQbsIGyr5msvI91cw36DIR2e0F3hm8xJTAjBgkqhkiG9w0B"
        "CRUxFgQUFP2Fd5wClCKW6tMEh+aNI6t6jcgwMTAhMAkGBSsOAwIaBQAEFM/eVvDz"
        "KM3yL/IUjt5H5LZohb2QBAhZFKH4Yqvt8gICCAA=";

static std::string targetFile = "filePath";
static std::string correctPassword_PKCS12 = "passwordpkcs12";
static int validStartYear = 114; // Since 1900 -> year 2014
//static int validEndYear = 211; // Since 1900 -> year 2111

class CertificateValidatorTest : public CPPUNIT_NS::TestFixture
{
    private:

    public:

        CertificateValidatorTest()
        {
        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testCertificateValidator_PEM_NullSafeBuffer ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = DefaultPurpose;

            CertificateValidator validator;
            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_CANNOT_READ_TARGET;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PEM_SelfSignedCert ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validSelfSignedCert_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = DefaultPurpose;

            CertificateValidator validator;
            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_DEPTH_ZERO_SELF_SIGNED_CERT;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PEM_SetOptions_DoNotValidate ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validSelfSignedCert_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = DefaultPurpose;
            CertificateValidationFlags validationFlags = VERIFY_DO_NOT_VALIDATE;

            CertificateValidator validator;
            validator.SetOptions ( validationFlags );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = true;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_OK;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PEM_ClearOptions_DoNotValidate ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validSelfSignedCert_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = DefaultPurpose;
            CertificateValidationFlags validationFlags = VERIFY_DO_NOT_VALIDATE;

            CertificateValidator validator;
            validator.SetOptions ( validationFlags );
            validator.ClearOptions ( validationFlags );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_DEPTH_ZERO_SELF_SIGNED_CERT;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        // TODO: VERIFY_IGNORE_CRITICAL one that fails
        void testCertificateValidator_PEM_SetCACertificate_PEM ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertified_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca.crt", targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = true;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_OK;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        // TODO: VERIFY_X509_STRICT one that fails
        void testCertificateValidator_PEM_SetCACertificate_DER ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertified_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca-der.crt", CertUtil::CertFormat_DER );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = true;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_OK;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        // TODO: change to receive something other than a path?
        void testCertificateValidator_PEM_SetCACertificateEmpty ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertified_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;

            CertificateValidator validator;
            validator.SetCACertificate ( "", targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_UNABLE_TO_GET_ISSUER_CERT_LOCALLY;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PEM_SetOptions_VerifySetCheckTime_SetVerificationTime_OK ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertified_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;
            CertificateValidationFlags validationFlags = VERIFY_SET_CHECK_TIME;
            time_t validationTime = time ( 0 );
            tm tm = *localtime ( &validationTime );
            tm.tm_year = validStartYear;
            time_t valTime = mktime ( &tm );

            CertificateValidator validator;
            validator.SetOptions ( validationFlags );
            validator.SetVerificationTime ( valTime );
            validator.SetCACertificate ( "root-ca.crt", targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = true;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_OK;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        // What does depth 1 mean?
        void testCertificateValidator_PEM_SetOptions_VerifySetCheckTime_SetVerificationTime_Fail ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertified_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;
            CertificateValidationFlags validationFlags = VERIFY_SET_CHECK_TIME;
            time_t validationTime = time ( 0 );
            tm tm = *localtime ( &validationTime );
            tm.tm_year = validStartYear - 2;
            time_t valTime = mktime ( &tm );

            CertificateValidator validator;
            validator.SetOptions ( validationFlags );
            validator.SetVerificationTime ( valTime );
            validator.SetCACertificate ( "root-ca.crt", targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 1;
            TrustResultEnum detailExpected = TRUST_CERT_NOT_YET_VALID;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PEM_ClearVerificationTime ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertified_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;
            CertificateValidationFlags validationFlags = VERIFY_SET_CHECK_TIME;
            time_t validationTime = time ( 0 );
            tm tm = *localtime ( &validationTime );
            tm.tm_year = validStartYear - 2;
            time_t valTime = mktime ( &tm );

            CertificateValidator validator;
            validator.SetOptions ( validationFlags );
            validator.SetVerificationTime ( valTime );
            validator.ClearVerificationTime ( );
            validator.SetCACertificate ( "root-ca.crt", targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = true;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_OK;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        // TODO: one where it does verify explicit policy successfully
        void testCertificateValidator_PEM_SetOptions_VerifyNotifyPolicy ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertified_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;
            CertificateValidationFlags validationFlags = VERIFY_EXPLICIT_POLICY;

            CertificateValidator validator;
            validator.SetOptions ( validationFlags );
            validator.SetCACertificate ( "root-ca.crt", targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_ERR_NO_EXPLICIT_POLICY;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        // TODO: one where the verification does work
        void testCertificateValidator_PEM_SetOptions_VerifyCBIssuerCheck ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertified_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;
            CertificateValidationFlags validationFlags = VERIFY_CB_ISSUER_CHECK;

            CertificateValidator validator;
            validator.SetOptions ( validationFlags );
            validator.SetCACertificate ( "root-ca.crt", targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_ERR_SUBJECT_ISSUER_MISMATCH;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        // TODO: one that succeeds
        void testCertificateValidator_PEM_SetOptions_VerifyCRLCheck ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertified_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;
            CertificateValidationFlags validationFlags = VERIFY_CRL_CHECK;

            CertificateValidator validator;
            validator.SetOptions ( validationFlags );
            validator.SetCACertificate ( "root-ca.crt", targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_ERR_UNABLE_TO_GET_CRL;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        // TODO: change to receive something other than a path?
        void testCertificateValidator_PEM_SetItermediateCACertificates ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertifiedWithIntermediateCA_PEM ), true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validIntermediateCA_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca.crt", targetFormat );
            validator.SetItermediateCACertificates ( targetFile, targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = true;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_OK;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PEM_SetItermediateCACertificates_CannotReadIntermediateCACerts ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertifiedWithIntermediateCA_PEM ), true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca.crt", targetFormat );
            validator.SetItermediateCACertificates ( targetFile, targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_UNABLE_TO_GET_ISSUER_CERT_LOCALLY;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_DER_SetItermediateCACertificates ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertifiedWithIntermediateCA_PEM ), true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( ), false ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca.crt", targetFormat );
            validator.SetItermediateCACertificates ( targetFile, CertUtil::CertFormat_DER );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_UNABLE_TO_GET_ISSUER_CERT_LOCALLY;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        // TODO: how do i do a trusted certificate? make another that succeeds
        void testCertificateValidator_PEM_SetTrustedCertificates_UnableToGetIssuerCert ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertifiedWithIntermediateCA_PEM ), true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validIntermediateCA_PEM ), true ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca.crt", targetFormat );
            validator.SetTrustedCertificates ( targetFile, targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 1;
            TrustResultEnum detailExpected = TRUST_UNABLE_TO_GET_ISSUER_CERT;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PEM_SetTrustedCertificates_CannotReadTrustedCerts ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( validCertifiedWithIntermediateCA_PEM ), true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( ), false ) );

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca.crt", targetFormat );
            validator.SetTrustedCertificates ( targetFile, targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, targetFormat, purpose );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_UNABLE_TO_GET_ISSUER_CERT_LOCALLY;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PEM_VerifyX509Certificate_OpenSSLCryptoX509 ()
        {
            SafeBuffer base64Cert ( validCertified_DER );
            OpenSSLCertificate cert ( base64Cert );
            OpenSSLCryptoX509 * key = cert.getX509();

            CertUtil::CertFileFormat targetFormat = CertUtil::CertFormat_PEM;
            CertificatePurpose purpose = AnyPurposeCertificate;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca.crt", targetFormat );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( key, purpose );

            bool successExpected = true;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_OK;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PKCS12_NullSafeBuffer ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );

            CertificatePurpose purpose = DefaultPurpose;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca.crt", CertUtil::CertFormat_PEM );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, CertUtil::CertFormat_PKCS12, purpose, correctPassword_PKCS12 );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_CANNOT_READ_TARGET;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        // TODO: see if all these flags are actually used for something
        void testCertificateValidator_PKCS12 ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertified_PKCS12, certificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            CertificatePurpose purpose = DefaultPurpose;
            CertificateValidationFlags validationFlags = ( CertificateValidationFlags )
                    ( VERIFY_CRL_CHECK_ALL | VERIFY_POLICY_CHECK
                      | VERIFY_INHIBIT_ANY | VERIFY_X509_STRICT | VERIFY_NOTIFY_POLICY
                      | VERIFY_CHECK_SS_SIGNATURE | VERIFY_ALLOW_PROXY_CERTS | VERIFY_IGNORE_CRITICAL );

            CertificateValidator validator;
            validator.SetOptions ( validationFlags );
            validator.SetCACertificate ( "root-ca.crt", CertUtil::CertFormat_PEM );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, CertUtil::CertFormat_PKCS12, purpose, correctPassword_PKCS12 );

            bool successExpected = true;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_OK;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PKCS12_SetItermediateCACertificates ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertifiedWithIntermediateCA_PKCS12, certificateBuffer );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            SafeBuffer intermediateCertificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validIntermediateCA_PKCS12, intermediateCertificateBuffer );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( intermediateCertificateBuffer, true ) );

            CertificatePurpose purpose = DefaultPurpose;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca.crt", CertUtil::CertFormat_PEM );
            validator.SetItermediateCACertificates ( targetFile, CertUtil::CertFormat_PKCS12, correctPassword_PKCS12 );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, CertUtil::CertFormat_PKCS12, purpose, correctPassword_PKCS12 );

            bool successExpected = true;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_OK;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        void testCertificateValidator_PKCS12_SetTrustedCertificates ()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertifiedWithIntermediateCA_PKCS12, certificateBuffer );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            SafeBuffer trustedCertificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validIntermediateCA_PKCS12, trustedCertificateBuffer );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( trustedCertificateBuffer, true ) );

            CertificatePurpose purpose = DefaultPurpose;

            CertificateValidator validator;
            validator.SetCACertificate ( "root-ca.crt", CertUtil::CertFormat_PEM );
            validator.SetTrustedCertificates ( targetFile, CertUtil::CertFormat_PKCS12, correctPassword_PKCS12 );

            CertValidationResult result;
            result =  validator.VerifyX509Certificate ( targetFile, CertUtil::CertFormat_PKCS12, purpose, correctPassword_PKCS12 );

            bool successExpected = false;
            int depthExpected = 0;
            TrustResultEnum detailExpected = TRUST_UNABLE_TO_GET_ISSUER_CERT;

            bool successActual = result.GetSucces();
            int depthActual = result.GetDepth();
            TrustResultEnum detailActual = result.GetDetail();

            CPPUNIT_ASSERT ( successExpected == successActual );
            CPPUNIT_ASSERT ( depthExpected == depthActual );
            CPPUNIT_ASSERT ( detailExpected == detailActual );
        }

        CPPUNIT_TEST_SUITE ( CertificateValidatorTest );

        CPPUNIT_TEST ( testCertificateValidator_PEM_NullSafeBuffer );
        CPPUNIT_TEST ( testCertificateValidator_PEM_SelfSignedCert );
        CPPUNIT_TEST ( testCertificateValidator_PEM_SetOptions_DoNotValidate );
        CPPUNIT_TEST ( testCertificateValidator_PEM_ClearOptions_DoNotValidate );

        CPPUNIT_TEST ( testCertificateValidator_PEM_SetCACertificate_PEM );
        CPPUNIT_TEST ( testCertificateValidator_PEM_SetCACertificate_DER );
        CPPUNIT_TEST ( testCertificateValidator_PEM_SetCACertificateEmpty );
        CPPUNIT_TEST ( testCertificateValidator_PEM_SetOptions_VerifySetCheckTime_SetVerificationTime_OK );
        CPPUNIT_TEST ( testCertificateValidator_PEM_SetOptions_VerifySetCheckTime_SetVerificationTime_Fail );
        CPPUNIT_TEST ( testCertificateValidator_PEM_ClearVerificationTime );

        CPPUNIT_TEST ( testCertificateValidator_PEM_SetOptions_VerifyNotifyPolicy );
        //CPPUNIT_TEST ( testCertificateValidator_PEM_SetOptions_VerifyCBIssuerCheck );
        CPPUNIT_TEST ( testCertificateValidator_PEM_SetOptions_VerifyCRLCheck );

        CPPUNIT_TEST ( testCertificateValidator_PEM_SetItermediateCACertificates );
        CPPUNIT_TEST ( testCertificateValidator_PEM_SetItermediateCACertificates_CannotReadIntermediateCACerts );
        CPPUNIT_TEST ( testCertificateValidator_DER_SetItermediateCACertificates );

        CPPUNIT_TEST ( testCertificateValidator_PEM_SetTrustedCertificates_UnableToGetIssuerCert );
        CPPUNIT_TEST ( testCertificateValidator_PEM_SetTrustedCertificates_CannotReadTrustedCerts );

        CPPUNIT_TEST ( testCertificateValidator_PEM_VerifyX509Certificate_OpenSSLCryptoX509 );

        CPPUNIT_TEST ( testCertificateValidator_PKCS12 );
        CPPUNIT_TEST ( testCertificateValidator_PKCS12_NullSafeBuffer );

        //CPPUNIT_TEST ( testCertificateValidator_PKCS12_SetItermediateCACertificates );
        //CPPUNIT_TEST ( testCertificateValidator_PKCS12_SetTrustedCertificates );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_TEST_CERTIFICATEVALIDATORTEST */
