package com.smartmatic.security;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.io.UnsupportedEncodingException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.zip.InflaterOutputStream;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.FileUtils;
import org.bouncycastle.crypto.generators.PKCS5S2ParametersGenerator;
import org.bouncycastle.crypto.params.KeyParameter;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

public class VoteDecryptor
{
	private static Map<Long, Map<Long, Long>> votesByContest;
	private static Map<Long, Long> votesByLists;
	private static Map<Long, Long> ballotOptionsListsMap;
	private static Map<Long, String> ballotOptions;
	private static Map<Long, String> partyInfo;

	private static byte[] generatePbeKey(byte[] salt, int iterationCount, String password)
			throws UnsupportedEncodingException
	{
		PKCS5S2ParametersGenerator parametersGenerator = new PKCS5S2ParametersGenerator();

		parametersGenerator.init(password.getBytes("UTF-8"), salt, 
				iterationCount);
		KeyParameter generateDerivedParameters = (KeyParameter)parametersGenerator
				.generateDerivedParameters(256);

		byte[] key = generateDerivedParameters.getKey();

		return key;
	}

	public static SecretKey getSymetricKey(byte[] decodedKey, String alg)
	{
		SecretKeySpec secretKey = new SecretKeySpec(decodedKey, alg);
		return secretKey;
	}

	private static MessageDigest getMessageDigest(int mode)
			throws NoSuchAlgorithmException
	{
		MessageDigest messageDigest;
		if (mode == 0)
		{
			messageDigest = MessageDigest.getInstance("SHA-1");
		}
		else
		{
			if (mode == 1) {
				messageDigest = MessageDigest.getInstance("MD5");
			} else {
				messageDigest = MessageDigest.getInstance("SHA-256");
			}
		}
		return messageDigest;
	}

	public static byte[] digest(byte[] message, int mode)
			throws NoSuchAlgorithmException
	{
		MessageDigest messageDigest = getMessageDigest(mode);
		messageDigest.reset();
		messageDigest.update(message);
		return messageDigest.digest();
	}

	public static byte[] decryptData(byte[] data, SecretKey pass, byte[] iVector)
			throws InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException
	{
		PoolableSymmetricCipher cipher = new PoolableSymmetricCipher(pass, 
				iVector);
		byte[] bytesDecrypted = cipher.decryptBytes(data);
		return bytesDecrypted;
	}

	public static String getProtectionKeyCBC(File pbeFile, String presidentKey, byte[] salt)
			throws NoSuchAlgorithmException, IOException, InvalidKeyException, NoSuchPaddingException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException
	{
		if (pbeFile.exists())
		{

			byte[] key = generatePbeKey(salt, 2048, presidentKey);

			byte[] decryptedData = getByteDecrypted(pbeFile, key);

			return 
					org.apache.commons.codec.binary.StringUtils.newStringUtf8(Base64.encodeBase64(decryptedData, true));
		}
		return "";
	}

	public static byte[] getByteDecrypted(File pbeFile, byte[] key)
			throws NoSuchAlgorithmException, IOException, InvalidKeyException, NoSuchPaddingException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException
	{
		SecretKey secretKey = getSymetricKey(key, "AES");

		Path path = Paths.get(pbeFile.getAbsolutePath());

		byte[] data = Files.readAllBytes(path);

		byte[] iVector = Arrays.copyOfRange(data, 0, 16);    

		byte[] dataToDecrypt = FileUtils.readFileToByteArray(pbeFile);

		byte[] decryptedData = decryptData(Arrays.copyOfRange(dataToDecrypt,16,dataToDecrypt.length), secretKey, iVector);

		return decryptedData;
	}

	/*public static void parseElection(byte[] bytesElection)
    throws DocumentException
  {
    String xml = new String(bytesElection)
      .replaceAll(
      "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>", 
      "").trim();

    Document document = DocumentHelper.parseText(xml);

    Element electionElement = document.getRootElement();

    List<Element> ballotsElements = electionElement.elements("ballots");
    Iterator localIterator2;
    for (Iterator localIterator1 = ballotsElements.iterator(); localIterator1.hasNext(); localIterator2.hasNext())
    {
      Element ballotsElement = (Element)localIterator1.next();

      List<Element> ballotElements = ballotsElement.elements("ballot");

      localIterator2 = ballotElements.iterator(); continue;
      Element ballotElement = (Element)localIterator2.next();

      List<Element> ballotCardsElements = ballotElement
        .elements("ballot-cards");
      Iterator localIterator4;
      for (Iterator localIterator3 = ballotCardsElements.iterator(); localIterator3.hasNext(); localIterator4.hasNext())
      {
        Element ballotCardsElement = (Element)localIterator3.next();

        List<Element> ballotCardElements = ballotCardsElement
          .elements("ballot-card");

        localIterator4 = ballotCardElements.iterator(); continue;Element ballotCardElement = (Element)localIterator4.next();

        List<Element> contestsElements = ballotCardElement
          .elements("contests");
        Iterator localIterator6;
        for (Iterator localIterator5 = contestsElements.iterator(); localIterator5.hasNext(); localIterator6.hasNext())
        {
          Element contestsElement = (Element)localIterator5.next();

          List<Element> contestElements = contestsElement
            .elements("contest");

          localIterator6 = contestElements.iterator(); continue;Element contestElement = (Element)localIterator6.next();

          long contestCode = Long.valueOf(contestElement
            .attributeValue("code")).longValue();

          List<Element> eligiblesOptionsElements = contestElement
            .elements("eligibles-options");
          Iterator localIterator8;
          for (Iterator localIterator7 = eligiblesOptionsElements.iterator(); localIterator7.hasNext(); localIterator8.hasNext())
          {
            Element eligiblesOptionsElement = (Element)localIterator7.next();

            List<Element> eligiblesOptionElements = eligiblesOptionsElement
              .elements("eligible-option");

            localIterator8 = eligiblesOptionElements.iterator(); continue;Element eligiblesOptionElement = (Element)localIterator8.next();

            String ballotName = eligiblesOptionElement
              .attributeValue("ballot-name-key");

            List<Element> ballotOptionsElements = eligiblesOptionElement
              .elements("ballot-options");
            Iterator localIterator10;
            for (Iterator localIterator9 = ballotOptionsElements.iterator(); localIterator9.hasNext(); localIterator10.hasNext())
            {
              Element ballotOptionsElement = (Element)localIterator9.next();

              List<Element> ballotOptionElements = ballotOptionsElement
                .elements("ballot-option");

              localIterator10 = ballotOptionElements.iterator(); continue;Element ballotOptionElement = (Element)localIterator10.next();

              long ballotOptionCode = 
                Long.valueOf(ballotOptionElement
                .attributeValue("code")).longValue();

              String tallyPosition = ballotOptionElement
                .attributeValue("tally-position");

              List<Element> partyElements = ballotOptionElement
                .elements("party");
              for (Element partyElement : partyElements)
              {
                String partyName = partyElement
                  .attributeValue("name");

                long partyCode = Long.valueOf(partyElement
                  .attributeValue("code")).longValue();

                String partyCustomCode = partyElement
                  .attributeValue("custom-code");

                ballotOptions.put(
                  Long.valueOf(ballotOptionCode), 
                  partyCustomCode + "|" + partyName.replace(' ', '_') + "|" + tallyPosition + "|" + ballotName);

                ballotOptionsListsMap.put(
                  Long.valueOf(ballotOptionCode), 
                  Long.valueOf(partyCode));

                partyInfo.put(
                  Long.valueOf(partyCode), 
                  partyCustomCode + "|" + partyName.replace(' ', '_'));
              }
            }
          }
        }
      }
    }
  }*/

	public static byte[] obtainSalt(byte[] bytesVotingCenter)
			throws DocumentException, NoSuchAlgorithmException
	{
		String xml = new String(bytesVotingCenter)  .replaceAll(
				"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>", 
				"").trim();
		String saltSeed="";
		Document document = DocumentHelper.parseText(xml);

		Element votingCenterRootElement = document.getRootElement();
		List<Element> votingCenterElements = votingCenterRootElement.elements("VotingCenter");
		for (Element votingCenterElement : votingCenterElements)
		{
			saltSeed = votingCenterElement.attributeValue("code").toLowerCase() + '-' + votingCenterElement.attributeValue("rnd");
			break;
		}
		MessageDigest messageDigest = getMessageDigest(2);
		messageDigest.reset();
		messageDigest.update(saltSeed.getBytes());
		return messageDigest.digest();
	}


	public static void parseVote(byte[] bytesVote)
			throws DocumentException
	{
		String xml = new String(bytesVote)
				.replaceAll(
						"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>", 
						"").trim();

		Document document = DocumentHelper.parseText(xml);

		Element voteElement = document.getRootElement();

		String demo = voteElement.attributeValue("m");
		if ((demo == null) || (demo.toUpperCase().equals("D")))
		{
			System.out.println("Invalid Vote -> demo atribute in true or is null");
			return;
		}
		List<Element> selectionElements = voteElement.elements("s");
		for (Element selectionElement : selectionElements)
		{
			long contestCode = Long.valueOf(selectionElement
					.attributeValue("c")).longValue();
			Map<Long, Long> votes;
			if (votesByContest.containsKey(Long.valueOf(contestCode))) {
				votes = (Map)votesByContest.get(Long.valueOf(contestCode));
			} else {
				votes = new HashMap();
			}
			if (contestCode != 0L)
			{
				List<Element> ballotElements = selectionElement.elements("b");
				if ((ballotElements == null) || (ballotElements.isEmpty()))
				{
					System.out.println("Blanc Vote Found");
				}
				else
				{
					boolean listAdded = false;
					for (Element ballotElement : ballotElements)
					{
						Long ballotOptionCode = Long.valueOf(ballotElement.getText());

						long voteAmountL = 1L;
						if ((!listAdded) && 
								(ballotOptionsListsMap.containsKey(ballotOptionCode)))
						{
							long partyCode = ((Long)ballotOptionsListsMap.get(ballotOptionCode)).longValue();
							if (votesByLists.containsKey(Long.valueOf(partyCode)))
							{
								voteAmountL = ((Long)votesByLists.get(Long.valueOf(partyCode))).longValue();
								voteAmountL += 1L;
							}
							votesByLists.put(Long.valueOf(partyCode), Long.valueOf(voteAmountL));
							listAdded = true;
						}
						long voteAmount = 1L;
						if (votes.containsKey(ballotOptionCode))
						{
							voteAmount = ((Long)votes.get(ballotOptionCode)).longValue();
							voteAmount += 1L;
						}
						votes.put(ballotOptionCode, Long.valueOf(voteAmount));
					}
					votesByContest.put(Long.valueOf(contestCode), votes);
				}
			}
		}
	}

	/*  public static void generateReport()
  {
    Set<Long> contestsCodes = votesByContest.keySet();
    System.out.println("**** CANDIDATES *****");
    Iterator iterator2;
    for (Iterator iterator = contestsCodes.iterator(); iterator.hasNext(); iterator2.hasNext())
    {
      Long contestCode = (Long)iterator.next();

      System.out.println("Contest Code = " + contestCode);

      Map<Long, Long> votes = (Map)votesByContest.get(contestCode);

      Set<Long> ballotOptionsCodes = votes.keySet();

      iterator2 = ballotOptionsCodes.iterator(); continue;

      //Long ballotOptionCode = (Long)iterator2.next();

      System.out.println(org.apache.commons.lang3.StringUtils.replace((String)ballotOptions.get(ballotOptionCode), "#", " ") + " = " + 
        votes.get(ballotOptionCode));
    }
    System.out.println("**** LISTS *****");

    Set<Long> listsCodes = votesByLists.keySet();
    for (Iterator iterator = listsCodes.iterator(); iterator.hasNext();)
    {
      Long listCode = (Long)iterator.next();

      System.out.println((String)partyInfo.get(listCode) + "|" + votesByLists.get(listCode));
    }
  }*/

	/**
	 * Uncompress Byte array.
	 * 
	 * @param decipheredData
	 *            Byte array with the file data compressed.
	 * @return Byte array with the file data uncompressed.
	 * @throws IOException
	 */
	private static byte[] uncompressByteArray(byte[] decipheredData) throws IOException {

		ByteArrayOutputStream out = new ByteArrayOutputStream();

		InflaterOutputStream inflaterOut = new InflaterOutputStream(out);

		try {
			inflaterOut.write(decipheredData);
		} finally {
			inflaterOut.finish();
			inflaterOut.close();
		}

		decipheredData = out.toByteArray();
		return decipheredData;
	}

	public static void main(String[] args) throws 
	InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, 
	InvalidAlgorithmParameterException, IllegalBlockSizeException,
	BadPaddingException, IOException, DocumentException
	{
		try
		{
			System.out.println("parsing arguments");

			String inputPath = args[0];
			String outputPath = args[1];
			String presidentPassphrase = args[2];

			System.out.println("input path:" + inputPath);
			System.out.println("output path:" + outputPath);
			System.out.println("president passphrase:" + presidentPassphrase);

			String initKeyDefault = "+DrhkiXyXx4v5jY3sUhZv9GSBr5xDdU7ZlEVrn/qPEI=";
			String SAESLocal = "/SAESLocal/";
			String votingCenterPath = inputPath + SAESLocal + "voting-centers.xml";
			String secretKeyPath = inputPath + SAESLocal + "secret-key";
			String upKeyPath = inputPath + SAESLocal + "up-key";
			String electionPath = inputPath + SAESLocal + "SAES/election.xml";
			String votingDevicePath = inputPath + SAESLocal + "SAES/voting-devices.xml";
			String votesFolderPath = inputPath + SAESLocal + "SAES/Votes/";
			String outputVotesFolderPath = outputPath + SAESLocal + "SAES/Votes/";
			String outputSaesFolderPath = outputPath + SAESLocal + "SAES/";

			File votingCenterFile = new File(votingCenterPath);
			File secretKeyFile = new File(secretKeyPath);
			File upKeyFile = new File(upKeyPath);
			File electionFile = new File(electionPath);
			File votingDeviceFile = new File(votingDevicePath);
			File votesFolder = new File(votesFolderPath);
			File[] listOfVoteFiles = votesFolder.listFiles();

			System.out.println("decoding voting center file in:" + votingCenterFile);
			byte[] votingCenterBuffer = getByteDecrypted(votingCenterFile, PoolableSymmetricCipher.decode(initKeyDefault));
			System.out.println("generating salt");
			byte[] salt = obtainSalt(votingCenterBuffer);
			System.out.println("obtaining Protection Key from:" + secretKeyFile);
			String protectionKeyCBC = getProtectionKeyCBC(secretKeyFile,presidentPassphrase,salt);
			System.out.println("obtaining UP Key from:" + upKeyFile);
			String upKeyCBC = getProtectionKeyCBC(upKeyFile,presidentPassphrase,salt);

			File outputVotesFolder = new File(outputVotesFolderPath);
			if (!outputVotesFolder.exists()) {
				System.out.println("creating output directory: " + outputVotesFolder.getAbsolutePath());
				outputVotesFolder.mkdir();     
			}

			System.out.println("trying to decrypt election.");
			byte[] decryptedElectionBuffer = getByteDecrypted(electionFile, PoolableSymmetricCipher.decode(upKeyCBC));
			System.out.println("uncompressing election buffer.");
			byte[] uncompressedElectionbuffer = uncompressByteArray(decryptedElectionBuffer);
			System.out.println("persisting vote file in:" + outputSaesFolderPath + electionFile.getName());
			FileUtils.writeByteArrayToFile(new File(outputSaesFolderPath + electionFile.getName()), uncompressedElectionbuffer);

			System.out.println("trying to decrypt voting device.");
			byte[] decryptedVotingDeviceBuffer = getByteDecrypted(votingDeviceFile, PoolableSymmetricCipher.decode(protectionKeyCBC));
			System.out.println("persisting voting device in:" + outputSaesFolderPath + votingDeviceFile.getName());
			FileUtils.writeByteArrayToFile(new File(outputSaesFolderPath + votingDeviceFile.getName()), decryptedVotingDeviceBuffer);

			System.out.println("trying to decrypt votes.");
			for (int i = 0; i < listOfVoteFiles.length; i++) {
				if (listOfVoteFiles[i].isFile()) {
					System.out.println("decrypting vote file:" + listOfVoteFiles[i].getAbsolutePath());
					byte[] decryptedVoteBuffer = getByteDecrypted(listOfVoteFiles[i], PoolableSymmetricCipher.decode(protectionKeyCBC));
					System.out.println("persisting vote file in:" + outputVotesFolderPath + listOfVoteFiles[i].getName());
					FileUtils.writeByteArrayToFile(new File(outputVotesFolderPath + listOfVoteFiles[i].getName()), decryptedVoteBuffer);
				} 
			}
		}
		catch (Throwable e)
		{
			System.err.println("There was an error: [caused by = "
					+ e.getMessage() + "]");
			System.out
			.println("Usage: java -jar VoteDecryptor.jar [inputFolderAbsolutePath] [outputFolderAbsolutePath] [presidentPassphrase]");
		}


	}
}

