#include <iostream>
#include <unistd.h>
#include <fstream>
#include <string>

#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/IO/Directory.h>
#include "TestsManager.hxx"

using namespace Smartmatic::Log;
using namespace Smartmatic::System::IO;

#ifdef CONFORMITY_TESTS_TOOL_CONFIG_DIR
static const std::string CONFORMITY_TESTS_CONFIG_PATH = CONFORMITY_TESTS_TOOL_CONFIG_DIR;
#else
static const std::string CONFORMITY_TESTS_CONFIG_PATH = ".";
#endif

static const std::string LOG_CONFIGURATION_FILE_NAME = "log4-configuration.xml";

static ISMTTLog* m_logger = SMTTLogManager::GetLogger("conformity-tests-tool", "conformity-tests-tool");

int main(int argc, char** argv)
{
    if ((argc > 0) && (std::string(argv[0]) == "-h"))
    {
        std::cerr << "Use: conformity-tests-tool FILE." << std::endl;
        std::cerr << "    Include electoral configuration, votes and votes results files in '" <<
            CONFORMITY_TESTS_CONFIG_PATH << "'." << std::endl;
        return 0;
    }

    const std::string logConfigurationFilePath = (CONFORMITY_TESTS_CONFIG_PATH == ".") ?
        LOG_CONFIGURATION_FILE_NAME : CONFORMITY_TESTS_CONFIG_PATH + "/" + LOG_CONFIGURATION_FILE_NAME;

    if (Directory::getInstance()->FileExists(logConfigurationFilePath) == false)
    {
        m_logger->Error("Do not remove log4-configuration.xml file from '" + CONFORMITY_TESTS_CONFIG_PATH + "'.");
        return 0;
    }

    SMTTLogManager::getInstance()->setConfigFile(logConfigurationFilePath);
    SMTTLogManager::getInstance()->Reconfigure();

    TestsManager testsManager;
    const bool electionFileFound    = testsManager.getElectionFileFound();
    const bool votesFileFound       = testsManager.getVotesFileFound();
    const bool voteResultsFileFound = testsManager.getVoteResultsFileFound();

    if (electionFileFound && (votesFileFound || voteResultsFileFound))
    {
        testsManager.initializeElectoralConfiguration();

        if (votesFileFound)
        {
            testsManager.initializeVotes();
        }

        if (voteResultsFileFound)
        {
            testsManager.initializeVoteResults();
        }

        testsManager.runConformityTests(votesFileFound, voteResultsFileFound);
    }

    return 0;
}

