#!/bin/bash -e
#
# File: smtt-compile-os-builder.sh
#
# This script compiles the third-party code and configurations
#
# Date: Tue 22 jun 2017 18:45:23
# Author: gustavo.garcia
#
# Revision 1
#
# Copyright 2017 Smartmatic Corp. All Rights Reserved.
#
# Confidential Information of Smartmatic Labs. Not for disclosure or distribution
# prior written consent. This software contains code, techniques and know-how which
# is confidential and proprietary to Smartmatic.
#
# Use of this software is subject to the terms of an end user license agreement.

OUTPUT_FOLDER="compiled-pkgs"
BASE_PATH=$(dirname $(readlink -f $0))

HELP="\n Compile OS dependencies and copy packages to \"$OUTPUT_FOLDER\" folder. The Jail used for cross-compiling will be the one set in \"TOOLCHAIN_SYSROOT\"\n
as environment variable. If the variable is not defined the compilation process will use /srv/chroot/i386/ as default folder.\n\n
Usage examples:\n
  $0 \n
  $0 -e \$CHROOT_FOLDER \n
Application Options:\n
  -h             \t\t-> Show this help\n
  -e \$folder    \t-> Install third-party-code packages on the specific jail/chroot folder.\n"

while getopts "he:" option
do
 case "${option}"
 in
 h) echo -e $HELP
    exit 0;;
 e) CHROOT_FOLDER=${OPTARG};;
 esac
done


if [ -d "$OUTPUT_FOLDER" ]; then
    rm -r $OUTPUT_FOLDER
fi

cd $BASE_PATH/../

for dir in third-party-code/*
do
    dir=${dir%*/}
    cd third-party-code/${dir##*/}
    echo "Compiling ${dir##*/}"
    ./scripts/create-release-makefiles.sh arch=i386
    ./scripts/make.sh
    ./scripts/create-deb-packages.sh

    if ! [ -z ${CHROOT_FOLDER} ] && [ "${dir##*/}" == "globalplatform" ]
    then
      echo -e "Extracting ${dir##*/} Package on $CHROOT_FOLDER\n"
      cp build/*.deb $CHROOT_FOLDER/root

cat <<EOT >> $CHROOT_FOLDER/root/run.sh
#!/bin/sh -e
dpkg -i --force-overwrite /root/*.deb
EOT
      chroot $CHROOT_FOLDER sh /root/run.sh

      rm $CHROOT_FOLDER/root/*.deb
    fi
    cd ../../
done

for dir in configuration-scripts/*
do
    dir=${dir%*/}
    cd configuration-scripts/${dir##*/}
    echo "Compiling ${dir##*/}"
    ./scripts/create-deb-packages.sh
    cd ../../
done

mkdir $OUTPUT_FOLDER
find . -name "*.deb"|grep build/|awk '{print "cp " $1 " compiled-pkgs/"}' | sh
#rm $OUTPUT_FOLDER/*-dev.deb


echo "Done"


