/**
 * CryptographicRuntimeException.java
 *
 * $Revision: 1.8 $ ($Date: 2011-04-06 04:47:27 $)
 *
 * Copyright (c) 2004 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */

package com.smartmatic.waffle.security.lang;

import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.lang.IExceptionInfo;

/**
 * <p>
 * Class used to indicate that an exception occurred because a problem with
 * related to cryptographic configuration as keys, initialization vectors, etc
 * occurred.
 * </p>
 * 
 * @version 1.0 (Feb 26, 2008)
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public class CryptographicRuntimeException extends BaseException {

	/**
	 * Holds the version of the class. It is used by the serialization engine to
	 * determine when a class its equivalent to another
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	private static final long serialVersionUID = 627749504991315476L;

	/**
	 * Enum that contains all the possible reasons because the
	 * CryptographicProviderStaticException can be thrown
	 * 
	 * @version 1.0 (26/02/2008)
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	public enum ExceptionInfo implements IExceptionInfo {
		CIPHER_POOL_EXTRACTION_PROBLEM("CIPHER_POOL_EXTRACTION_PROBLEM"), ILEGAL_BLOCK_PROBLEM(
				"ILEGAL_BLOCK_PROBLEM"), PADDING_PROBLEM("PADDING_PROBLEM"), ASYMMETRIC_KEY_PROBLEM(
				"ASYMMETRIC_KEY_PROBLEM"), SYMMETRIC_KEY_PROBLEM(
				"SYMMETRIC_KEY_PROBLEM"), UNCATEGORIZED_PROBLEM(
				"UNCATEGORIZED_PROBLEM"), IO_PROBLEM("IO_PROBLEM"), INVALID_MESSAGE_FORMAT(
				"INVALID_MESSAGE_FORMAT"), INVALID_CERTIFICATE_FORMAT(
				"INVALID_CERTIFICATE_FORMAT");

		/**
		 * <p>
		 * Each enumerator's resource bundle's key message.
		 * </p>
		 */

		private String message;

		/**
		 * Getter of <code>message</code> property.
		 * 
		 * @return <code>message</code> property value.
		 * @see #message
		 * 
		 * @author Ithamar Torres (itorres@smartmatic.com)
		 */

		@Override
		public String getMessage() {
			return this.message;
		}

		/**
		 * Constructor
		 * 
		 * @param msg
		 *            the resource bundle's key message
		 * 
		 * @author Ithamar Torres (itorres@smartmatic.com)
		 */

		private ExceptionInfo(String msg) {
			this.message = this.getClass().getName() + "." + msg;
		}
	}

	/**
	 * Creates a instance of a CryptographicRuntimeException
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	public CryptographicRuntimeException(IExceptionInfo exceptionInfo) {
		super(exceptionInfo);
		return;
	}

	/**
	 * Creates a instance of a CryptographicRuntimeException
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @param e
	 *            a nested exception
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	public CryptographicRuntimeException(IExceptionInfo exceptionInfo,
			Throwable e) {
		super(exceptionInfo, e);
		return;
	}

	/**
	 * Creates a instance of a CryptographicRuntimeException
	 * 
	 * @param exceptionInfo
	 *            The reason because the exception is being thrown
	 * @param parameters
	 *            params used to give information about the exception
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */
	public CryptographicRuntimeException(IExceptionInfo exceptionInfo,
			Object... parameters) {
		super(exceptionInfo, parameters);
		return;
	}

}
